# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Andrew Colin Kissa <andrew@topdog.za.net>`
'''
from __future__ import absolute_import

from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

from salt.states import postgres_privileges


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PostgresPrivilegesTestCase(TestCase):
    '''
    Test cases for salt.states.postgres_privileges
    '''

    def setUp(self):
        '''
        Setup data for the tests
        '''
        postgres_privileges.__opts__ = {}
        postgres_privileges.__salt__ = {}
        self.table_name = 'awl'
        self.group_name = 'admins'
        self.name = 'baruwa'
        self.ret = {'name': self.name,
               'changes': {},
               'result': False,
               'comment': ''}
        self.mock_true = MagicMock(return_value=True)
        self.mock_false = MagicMock(return_value=False)

    def test_present_table(self):
        '''
        Test present
        '''
        with patch.dict(postgres_privileges.__salt__,
                {'postgres.has_privileges': self.mock_true}):
            comt = 'The requested privilege(s) are already set'
            self.ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(
                postgres_privileges.present(
                    self.name,
                    self.table_name,
                    'table'),
                self.ret)

        with patch.dict(postgres_privileges.__salt__,
            {'postgres.has_privileges': self.mock_false,
                'postgres.privileges_grant': self.mock_true}):
            with patch.dict(postgres_privileges.__opts__, {'test': True}):
                comt = ('The privilege(s): {0} are'
                        ' set to be granted to {1}').format('ALL', self.name)
                self.ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(
                    postgres_privileges.present(self.name,
                        self.table_name, 'table', privileges=['ALL']), self.ret)

            with patch.dict(postgres_privileges.__opts__, {'test': False}):
                comt = ('The privilege(s): {0} have '
                        'been granted to {1}').format('ALL', self.name)
                self.ret.update({'comment': comt,
                            'result': True,
                            'changes': {'baruwa': 'Present'}})
                self.assertDictEqual(
                    postgres_privileges.present(self.name,
                        self.table_name, 'table', privileges=['ALL']), self.ret)

    def test_present_group(self):
        '''
        Test present group
        '''
        with patch.dict(postgres_privileges.__salt__,
            {'postgres.has_privileges': self.mock_false,
                'postgres.privileges_grant': self.mock_true}):
            with patch.dict(postgres_privileges.__opts__, {'test': True}):
                comt = ('The privilege(s): {0} are'
                        ' set to be granted to {1}').format(self.group_name,
                            self.name)
                self.ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(
                    postgres_privileges.present(self.name,
                        self.group_name, 'group'), self.ret)

            with patch.dict(postgres_privileges.__opts__, {'test': False}):
                comt = ('The privilege(s): {0} have '
                        'been granted to {1}').format(self.group_name,
                            self.name)
                self.ret.update({'comment': comt,
                            'result': True,
                            'changes': {'baruwa': 'Present'}})
                self.assertDictEqual(
                    postgres_privileges.present(self.name,
                        self.group_name, 'group'), self.ret)

    def test_absent_table(self):
        '''
        Test absent
        '''
        with patch.dict(postgres_privileges.__salt__,
                {'postgres.has_privileges': self.mock_false}):
            with patch.dict(postgres_privileges.__opts__, {'test': True}):
                comt = ('The requested privilege(s)'
                    ' are not set so cannot be revoked')
                self.ret.update({'comment': comt, 'result': True})
                self.assertDictEqual(
                    postgres_privileges.absent(
                        self.name,
                        self.table_name,
                        'table'),
                    self.ret)

        with patch.dict(postgres_privileges.__salt__,
            {'postgres.has_privileges': self.mock_true,
                'postgres.privileges_revoke': self.mock_true}):
            with patch.dict(postgres_privileges.__opts__, {'test': True}):
                comt = ('The privilege(s): {0} are'
                        ' set to be revoked from {1}').format('ALL', self.name)
                self.ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(
                    postgres_privileges.absent(self.name,
                        self.table_name, 'table', privileges=['ALL']), self.ret)

            with patch.dict(postgres_privileges.__opts__, {'test': False}):
                comt = ('The privilege(s): {0} have '
                        'been revoked from {1}').format('ALL', self.name)
                self.ret.update({'comment': comt,
                            'result': True,
                            'changes': {'baruwa': 'Absent'}})
                self.assertDictEqual(
                    postgres_privileges.absent(self.name,
                        self.table_name, 'table', privileges=['ALL']), self.ret)

    def test_absent_group(self):
        '''
        Test absent group
        '''
        with patch.dict(postgres_privileges.__salt__,
            {'postgres.has_privileges': self.mock_true,
                'postgres.privileges_revoke': self.mock_true}):
            with patch.dict(postgres_privileges.__opts__, {'test': True}):
                comt = ('The privilege(s): {0} are'
                        ' set to be revoked from {1}').format(self.group_name,
                            self.name)
                self.ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(
                    postgres_privileges.absent(self.name,
                        self.group_name, 'group'), self.ret)

            with patch.dict(postgres_privileges.__opts__, {'test': False}):
                comt = ('The privilege(s): {0} have '
                        'been revoked from {1}').format(self.group_name,
                            self.name)
                self.ret.update({'comment': comt,
                            'result': True,
                            'changes': {'baruwa': 'Absent'}})
                self.assertDictEqual(
                    postgres_privileges.absent(self.name,
                        self.group_name, 'group'), self.ret)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import hg

# Globals
hg.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class HgTestCase(TestCase):
    '''
    Test cases for salt.modules.hg
    '''
    def test_revision(self):
        '''
        Test for Returns the long hash of a given identifier
        '''
        mock = MagicMock(side_effect=[{'retcode': 0, 'stdout': 'A'},
                                        {'retcode': 1, 'stdout': 'A'}])
        with patch.dict(hg.__salt__, {'cmd.run_all': mock}):
            self.assertEqual(hg.revision('cwd'), 'A')

            self.assertEqual(hg.revision('cwd'), '')

    def test_describe(self):
        '''
        Test for Mimic git describe.
        '''
        with patch.dict(hg.__salt__, {'cmd.run_stdout':
                                        MagicMock(return_value='A')}):
            with patch.object(hg, 'revision', return_value=False):
                self.assertEqual(hg.describe('cwd'), 'A')

    def test_archive(self):
        '''
        Test for Export a tarball from the repository
        '''
        with patch.dict(hg.__salt__, {'cmd.run':
                                      MagicMock(return_value='A')}):
            self.assertEqual(hg.archive('cwd', 'output'), 'A')

    def test_pull(self):
        '''
        Test for Perform a pull on the given repository
        '''
        with patch.dict(hg.__salt__, {'cmd.run_all':
                                      MagicMock(return_value={'retcode': 0,
                                                              'stdout': 'A'})}):
            self.assertEqual(hg.pull('cwd'), 'A')

    def test_update(self):
        '''
        Test for Update to a given revision
        '''
        with patch.dict(hg.__salt__, {'cmd.run_all':
                                      MagicMock(return_value={'retcode': 0,
                                                              'stdout': 'A'})}):
            self.assertEqual(hg.update('cwd', 'rev'), 'A')

    def test_clone(self):
        '''
        Test for Clone a new repository
        '''
        with patch.dict(hg.__salt__, {'cmd.run_all':
                                      MagicMock(return_value={'retcode': 0,
                                                              'stdout': 'A'})}):
            self.assertEqual(hg.clone('cwd', 'repository'), 'A')

    def test_status_single(self):
        '''
        Test for Status to a given repository
        '''
        with patch.dict(hg.__salt__, {'cmd.run_stdout':
                                        MagicMock(return_value=(
                                            'A added 0\n'
                                            'A added 1\n'
                                            'M modified'))}):
            self.assertEqual(hg.status('cwd'), {
                'added': ['added 0', 'added 1'],
                'modified': ['modified'],
            })

    def test_status_multiple(self):
        '''
        Test for Status to a given repository (cwd is list)
        '''
        with patch.dict(hg.__salt__, {
            'cmd.run_stdout': MagicMock(side_effect=(
                lambda *args, **kwargs: {
                    'dir 0': 'A file 0\n',
                    'dir 1': 'M file 1'
                }[kwargs['cwd']]))}):
            self.assertEqual(hg.status(['dir 0', 'dir 1']), {
                'dir 0': {'added': ['file 0']},
                'dir 1': {'modified': ['file 1']},
            })


if __name__ == '__main__':
    from integration import run_tests
    run_tests(HgTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Pyrax Cloud Module
==================

PLEASE NOTE: This module is currently in early development, and considered to
be experimental and unstable. It is not recommended for production use. Unless
you are actively developing code in this module, you should use the OpenStack
module instead.
'''

# Import Python Libs
from __future__ import absolute_import

# Import salt libs
import salt.utils
import salt.config as config

# Import pyrax libraries
# This is typically against SaltStack coding styles,
# it should be 'import salt.utils.openstack.pyrax as suop'.  Something
# in the loader is creating a name clash and making that form fail
from salt.utils.openstack import pyrax as suop

__virtualname__ = 'pyrax'


# Only load in this module is the PYRAX configurations are in place
def __virtual__():
    '''
    Check for Pyrax configurations
    '''
    if get_configured_provider() is False:
        return False

    if get_dependencies() is False:
        return False

    return __virtualname__


def get_configured_provider():
    '''
    Return the first configured instance.
    '''
    return config.is_provider_configured(
        __opts__,
        __active_provider_name__ or __virtualname__,
        ('username', 'identity_url', 'compute_region',)
    )


def get_dependencies():
    '''
    Warn if dependencies aren't met.
    '''
    return config.check_driver_dependencies(
        __virtualname__,
        {'pyrax': suop.HAS_PYRAX}
    )


def get_conn(conn_type):
    '''
    Return a conn object for the passed VM data
    '''
    vm_ = get_configured_provider()

    kwargs = vm_.copy()  # pylint: disable=E1103

    kwargs['username'] = vm_['username']
    kwargs['auth_endpoint'] = vm_.get('identity_url', None)
    kwargs['region'] = vm_['compute_region']

    conn = getattr(suop, conn_type)

    return conn(**kwargs)


def queues_exists(call, kwargs):
    conn = get_conn('RackspaceQueues')
    return conn.exists(kwargs['name'])


def queues_show(call, kwargs):
    conn = get_conn('RackspaceQueues')
    return salt.utils.simple_types_filter(conn.show(kwargs['name']).__dict__)


def queues_create(call, kwargs):
    conn = get_conn('RackspaceQueues')
    if conn.create(kwargs['name']):
        return salt.utils.simple_types_filter(conn.show(kwargs['name']).__dict__)
    else:
        return {}


def queues_delete(call, kwargs):
    conn = get_conn('RackspaceQueues')
    if conn.delete(kwargs['name']):
        return {}
    else:
        return salt.utils.simple_types_filter(conn.show(kwargs['name'].__dict__))



# -*- coding: utf-8 -*-
'''
ACME / Let's Encrypt certificate management state
=================================================

.. versionadded: 2016.3

See also the module documentation

.. code-block:: yaml

    reload-gitlab:
      cmd.run:
        - name: gitlab-ctl hup

    dev.example.com:
      acme.cert:
        - aliases:
          - gitlab.example.com
        - email: acmemaster@example.com
        - webroot: /opt/gitlab/embedded/service/gitlab-rails/public
        - renew: 14
        - fire_event: acme/dev.example.com
        - onchanges_in:
          - cmd: reload-gitlab

'''
# Import python libs
from __future__ import absolute_import
import logging

log = logging.getLogger(__name__)


def __virtual__():
    '''
    Only work when the ACME module agrees
    '''
    return 'acme.cert' in __salt__


def cert(name,
         aliases=None,
         email=None,
         webroot=None,
         test_cert=False,
         renew=None,
         keysize=None,
         server=None,
         owner='root',
         group='root'):
    '''
    Obtain/renew a certificate from an ACME CA, probably Let's Encrypt.

    :param name: Common Name of the certificate (DNS name of certificate)
    :param aliases: subjectAltNames (Additional DNS names on certificate)
    :param email: e-mail address for interaction with ACME provider
    :param webroot: True or full path to webroot used for authentication
    :param test_cert: Request a certificate from the Happy Hacker Fake CA (mutually exclusive with 'server')
    :param renew: True/'force' to force a renewal, or a window of renewal before expiry in days
    :param keysize: RSA key bits
    :param server: API endpoint to talk to
    :param owner: owner of private key
    :param group: group of private key
    '''

    if __opts__['test']:
        ret = {
            'name': name,
            'changes': {},
            'result': None
        }
        window = None
        try:
            window = int(renew)
        except:  # pylint: disable=bare-except
            pass

        comment = 'Certificate {0} '.format(name)
        if not __salt__['acme.has'](name):
            comment += 'would have been obtained'
        elif __salt__['acme.needs_renewal'](name, window):
            comment += 'would have been renewed'
        else:
            comment += 'would not have been touched'
        ret['comment'] = comment
        return ret

    if not __salt__['acme.has'](name):
        old = None
    else:
        old = __salt__['acme.info'](name)

    res = __salt__['acme.cert'](
        name,
        aliases=aliases,
        email=email,
        webroot=webroot,
        test_cert=test_cert,
        renew=renew,
        keysize=keysize,
        server=server,
        owner=owner,
        group=group
    )

    ret = {
        'name': name,
        'result': res['result'] is not False,
        'comment': res['comment']
    }

    if res['result'] is None:
        ret['changes'] = {}
    else:
        ret['changes'] = {
            'old': old,
            'new': __salt__['acme.info'](name)
        }

    return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import ilo

# Globals
ilo.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class IloTestCase(TestCase):
    '''
    Test cases for salt.modules.ilo
    '''
    # 'global_settings' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Global Settings': {}}))
    def test_global_settings(self):
        '''
        Test if it shows global_settings
        '''
        self.assertDictEqual(ilo.global_settings(), {'Global Settings': {}})

    # 'set_http_port' function tests: 1

    def test_set_http_port(self):
        '''
        Test if it configure the port HTTP should listen on
        '''
        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'HTTP_PORT':
                                                            {'VALUE': 80}}}):
            self.assertTrue(ilo.set_http_port())

        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'HTTP_PORT':
                                                            {'VALUE': 40}}}):
            with patch.object(ilo, '__execute_cmd',
                              return_value={'Set HTTP Port': {}}):
                self.assertDictEqual(ilo.set_http_port(),
                                     {'Set HTTP Port': {}})

    # 'set_https_port' function tests: 1

    def test_set_https_port(self):
        '''
        Test if it configure the port HTTPS should listen on
        '''
        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'HTTP_PORT':
                                                            {'VALUE': 443}}}):
            self.assertTrue(ilo.set_https_port())

        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'HTTP_PORT':
                                                            {'VALUE': 80}}}):
            with patch.object(ilo, '__execute_cmd',
                              return_value={'Set HTTPS Port': {}}):
                self.assertDictEqual(ilo.set_https_port(),
                                     {'Set HTTPS Port': {}})

    # 'enable_ssh' function tests: 1

    def test_enable_ssh(self):
        '''
        Test if it enable the SSH daemon
        '''
        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'SSH_STATUS':
                                                            {'VALUE': 'Y'}}}):
            self.assertTrue(ilo.enable_ssh())

        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'SSH_STATUS':
                                                            {'VALUE': 'N'}}}):
            with patch.object(ilo, '__execute_cmd',
                              return_value={'Enable SSH': {}}):
                self.assertDictEqual(ilo.enable_ssh(), {'Enable SSH': {}})

    # 'disable_ssh' function tests: 1

    def test_disable_ssh(self):
        '''
        Test if it disable the SSH daemon
        '''
        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'SSH_STATUS':
                                                            {'VALUE': 'N'}}}):
            self.assertTrue(ilo.disable_ssh())

        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'SSH_STATUS':
                                                            {'VALUE': 'Y'}}}):
            with patch.object(ilo, '__execute_cmd',
                              return_value={'Disable SSH': {}}):
                self.assertDictEqual(ilo.disable_ssh(), {'Disable SSH': {}})

    # 'set_ssh_port' function tests: 1

    def test_set_ssh_port(self):
        '''
        Test if it enable SSH on a user defined port
        '''
        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'SSH_PORT':
                                                            {'VALUE': 22}}}):
            self.assertTrue(ilo.set_ssh_port())

        with patch.object(ilo, 'global_settings',
                          return_value={'Global Settings': {'SSH_PORT':
                                                            {'VALUE': 20}}}):
            with patch.object(ilo, '__execute_cmd',
                              return_value={'Configure SSH Port': {}}):
                self.assertDictEqual(ilo.set_ssh_port(),
                                     {'Configure SSH Port': {}})

    # 'set_ssh_key' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Import SSH Publickey': {}}))
    def test_set_ssh_key(self):
        '''
        Test if it configure SSH public keys for specific users
        '''
        self.assertDictEqual(ilo.set_ssh_key('ssh-rsa AAAAB3Nza Salt'),
                             {'Import SSH Publickey': {}})

    # 'delete_ssh_key' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Delete user SSH key': {}}))
    def test_delete_ssh_key(self):
        '''
        Test if it delete a users SSH key from the ILO
        '''
        self.assertDictEqual(ilo.delete_ssh_key('Salt'),
                             {'Delete user SSH key': {}})

    # 'list_users' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'All users': {}}))
    def test_list_users(self):
        '''
        Test if it list all users
        '''
        self.assertDictEqual(ilo.list_users(), {'All users': {}})

    # 'list_users_info' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'All users info': {}}))
    def test_list_users_info(self):
        '''
        Test if it List all users in detail
        '''
        self.assertDictEqual(ilo.list_users_info(), {'All users info': {}})

    # 'create_user' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Create user': {}}))
    def test_create_user(self):
        '''
        Test if it create user
        '''
        self.assertDictEqual(ilo.create_user('Salt', 'secretagent',
                                             'VIRTUAL_MEDIA_PRIV'),
                             {'Create user': {}})

    # 'delete_user' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Delete user': {}}))
    def test_delete_user(self):
        '''
        Test if it delete a user
        '''
        self.assertDictEqual(ilo.delete_user('Salt'), {'Delete user': {}})

    # 'get_user' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'User Info': {}}))
    def test_get_user(self):
        '''
        Test if it returns local user information, excluding the password
        '''
        self.assertDictEqual(ilo.get_user('Salt'), {'User Info': {}})

    # 'change_username' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Change username': {}}))
    def test_change_username(self):
        '''
        Test if it change a username
        '''
        self.assertDictEqual(ilo.change_username('Salt', 'SALT'),
                             {'Change username': {}})

    # 'change_password' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Change password': {}}))
    def test_change_password(self):
        '''
        Test if it reset a users password
        '''
        self.assertDictEqual(ilo.change_password('Salt', 'saltpasswd'),
                             {'Change password': {}})

    # 'network' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Network Settings': {}}))
    def test_network(self):
        '''
        Test if it grab the current network settings
        '''
        self.assertDictEqual(ilo.network(), {'Network Settings': {}})

    # 'configure_network' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Configure_Network': {}}))
    def test_configure_network(self):
        '''
        Test if it configure Network Interface
        '''
        ret = {'Network Settings':
               {'IP_ADDRESS': {'VALUE': '10.0.0.10'},
                'SUBNET_MASK': {'VALUE': '255.255.255.0'},
                'GATEWAY_IP_ADDRESS': {'VALUE': '10.0.0.1'}}}
        with patch.object(ilo, 'network', return_value=ret):
            self.assertTrue(ilo.configure_network('10.0.0.10',
                                                  '255.255.255.0', '10.0.0.1'))

        with patch.object(ilo, 'network', return_value=ret):
            with patch.object(ilo, '__execute_cmd',
                              return_value={'Network Settings': {}}):
                self.assertDictEqual(ilo.configure_network('10.0.0.100',
                                                           '255.255.255.10',
                                                           '10.0.0.10'),
                                     {'Network Settings': {}})

    # 'enable_dhcp' function tests: 1

    def test_enable_dhcp(self):
        '''
        Test if it enable DHCP
        '''
        with patch.object(ilo, 'network',
                          return_value={'Network Settings': {'DHCP_ENABLE':
                                                             {'VALUE': 'Y'}}}):
            self.assertTrue(ilo.enable_dhcp())

        with patch.object(ilo, 'network',
                          return_value={'Network Settings': {'DHCP_ENABLE':
                                                             {'VALUE': 'N'}}}):
            with patch.object(ilo, '__execute_cmd',
                              return_value={'Enable DHCP': {}}):
                self.assertDictEqual(ilo.enable_dhcp(), {'Enable DHCP': {}})

    # 'disable_dhcp' function tests: 1

    def test_disable_dhcp(self):
        '''
        Test if it disable DHCP
        '''
        with patch.object(ilo, 'network',
                          return_value={'Network Settings': {'DHCP_ENABLE':
                                                             {'VALUE': 'N'}}}):
            self.assertTrue(ilo.disable_dhcp())

        with patch.object(ilo, 'network',
                          return_value={'Network Settings': {'DHCP_ENABLE':
                                                             {'VALUE': 'Y'}}}):
            with patch.object(ilo, '__execute_cmd',
                              return_value={'Disable DHCP': {}}):
                self.assertDictEqual(ilo.disable_dhcp(), {'Disable DHCP': {}})

    # 'configure_snmp' function tests: 1

    @patch('salt.modules.ilo.__execute_cmd',
           MagicMock(return_value={'Configure SNMP': {}}))
    def test_configure_snmp(self):
        '''
        Test if it configure SNMP
        '''
        self.assertDictEqual(ilo.configure_snmp('Salt'), {'Configure SNMP': {}})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(IloTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    mock_open,
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import grub_legacy
from salt.exceptions import CommandExecutionError

# Globals
grub_legacy.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GrublegacyTestCase(TestCase):
    '''
    Test cases for salt.modules.grub_legacy
    '''
    def test_version(self):
        '''
        Test for Return server version from grub --version
        '''
        mock = MagicMock(return_value='out')
        with patch.dict(grub_legacy.__salt__, {'cmd.run': mock}):
            self.assertEqual(grub_legacy.version(), 'out')

    def test_conf(self):
        '''
        Test for Parse GRUB conf file
        '''
        mock = MagicMock(side_effect=IOError('foo'))
        with patch('salt.utils.fopen', mock):
            with patch.object(grub_legacy, '_detect_conf', return_value='A'):
                self.assertRaises(CommandExecutionError, grub_legacy.conf)

        file_data = '\n'.join(['#', 'A B C D,E,F G H'])
        with patch('salt.utils.fopen',
                   mock_open(read_data=file_data), create=True) as f_mock:
            f_mock.return_value.__iter__.return_value = file_data.splitlines()
            with patch.object(grub_legacy, '_detect_conf', return_value='A'):
                self.assertEqual(grub_legacy.conf(),
                                 {'A': 'B C D,E,F G H', 'stanzas': []})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GrublegacyTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
These only test the provider selection and verification logic, they do not init
any remotes.
'''

# Import python libs
from __future__ import absolute_import

# Import Salt Testing libs
from salttesting import skipIf, TestCase
from salttesting.mock import MagicMock, patch, NO_MOCK, NO_MOCK_REASON
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import salt.utils.gitfs
from salt.exceptions import FileserverConfigError

# GLOBALS
OPTS = {'cachedir': '/tmp/gitfs-test-cache'}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class TestGitFSProvider(TestCase):

    def test_provider_case_insensitive(self):
        '''
        Ensure that both lowercase and non-lowercase values are supported
        '''
        provider = 'GitPython'
        for role_name, role_class in (
                ('gitfs', salt.utils.gitfs.GitFS),
                ('git_pillar', salt.utils.gitfs.GitPillar),
                ('winrepo', salt.utils.gitfs.WinRepo)):

            key = '{0}_provider'.format(role_name)
            with patch.object(role_class, 'verify_gitpython',
                              MagicMock(return_value=True)):
                with patch.object(role_class, 'verify_pygit2',
                                  MagicMock(return_value=False)):
                    with patch.object(role_class, 'verify_dulwich',
                                      MagicMock(return_value=False)):
                        args = [OPTS]
                        if role_name == 'winrepo':
                            args.append('/tmp/winrepo-dir')
                        with patch.dict(OPTS, {key: provider}):
                            # Try to create an instance with uppercase letters in
                            # provider name. If it fails then a
                            # FileserverConfigError will be raised, so no assert is
                            # necessary.
                            role_class(*args)
                            # Now try to instantiate an instance with all lowercase
                            # letters. Again, no need for an assert here.
                            role_class(*args)

    def test_valid_provider(self):
        '''
        Ensure that an invalid provider is not accepted, raising a
        FileserverConfigError.
        '''
        def _get_mock(verify, provider):
            '''
            Return a MagicMock with the desired return value
            '''
            return MagicMock(return_value=verify.endswith(provider))

        for role_name, role_class in (
                ('gitfs', salt.utils.gitfs.GitFS),
                ('git_pillar', salt.utils.gitfs.GitPillar),
                ('winrepo', salt.utils.gitfs.WinRepo)):
            key = '{0}_provider'.format(role_name)
            for provider in salt.utils.gitfs.VALID_PROVIDERS:
                verify = 'verify_gitpython'
                mock1 = _get_mock(verify, provider)
                with patch.object(role_class, verify, mock1):
                    verify = 'verify_pygit2'
                    mock2 = _get_mock(verify, provider)
                    with patch.object(role_class, verify, mock2):
                        verify = 'verify_dulwich'
                        mock3 = _get_mock(verify, provider)
                        with patch.object(role_class, verify, mock3):
                            args = [OPTS]
                            if role_name == 'winrepo':
                                args.append('/tmp/winrepo-dir')
                            with patch.dict(OPTS, {key: provider}):
                                if role_name == 'gitfs' \
                                        or (role_name != 'gitfs'
                                            and provider != 'dulwich'):
                                    # This is a valid provider, so this should
                                    # pass without raising an exception.
                                    role_class(*args)
                                else:
                                    # Dulwich is not supported for git_pillar nor
                                    # winrepo, so trying to use it should raise an
                                    # exception.
                                    self.assertRaises(
                                        FileserverConfigError,
                                        role_class,
                                        *args
                                    )

                            with patch.dict(OPTS, {key: 'foo'}):
                                # Set the provider name to a known invalid provider
                                # and make sure it raises an exception.
                                self.assertRaises(
                                    FileserverConfigError,
                                    role_class,
                                    *args
                                )


if __name__ == '__main__':
    from integration import run_tests
    run_tests(TestGitFSProvider, needs_daemon=False)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import
import errno
import os

# Import Salt Testing libs
from salttesting import TestCase, skipIf
from salttesting.mock import Mock, patch, NO_MOCK, NO_MOCK_REASON

# Import Salt libs
from salt.exceptions import SaltInvocationError
from salt.utils import systemd as _systemd


def _booted_effect(path):
    return True if path == '/run/systemd/system' else os.stat(path)


def _not_booted_effect(path):
    if path == '/run/systemd/system':
        raise OSError(errno.ENOENT, 'No such file or directory', path)
    return os.stat(path)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SystemdTestCase(TestCase):
    '''
    Tests the functions in salt.utils.systemd
    '''
    def test_booted(self):
        '''
        Test that salt.utils.systemd.booted() returns True when minion is
        systemd-booted.
        '''
        # Ensure that os.stat returns True. os.stat doesn't return a bool
        # normally, but the code is doing a simple truth check on the return
        # data, so it is sufficient enough to mock it as True for these tests.
        with patch('os.stat', side_effect=_booted_effect):
            # Test without context dict passed
            self.assertTrue(_systemd.booted())
            # Test that context key is set when context dict is passed
            context = {}
            self.assertTrue(_systemd.booted(context))
            self.assertEqual(context, {'salt.utils.systemd.booted': True})

    def test_not_booted(self):
        '''
        Test that salt.utils.systemd.booted() returns False when minion is not
        systemd-booted.
        '''
        # Ensure that os.stat raises an exception even if test is being run on
        # a systemd-booted host.
        with patch('os.stat', side_effect=_not_booted_effect):
            # Test without context dict passed
            self.assertFalse(_systemd.booted())
            # Test that context key is set when context dict is passed
            context = {}
            self.assertFalse(_systemd.booted(context))
            self.assertEqual(context, {'salt.utils.systemd.booted': False})

    def test_booted_return_from_context(self):
        '''
        Test that the context data is returned when present. To ensure we're
        getting data from the context dict, we use a non-boolean value to
        differentiate it from the True/False return this function normally
        produces.
        '''
        context = {'salt.utils.systemd.booted': 'foo'}
        self.assertEqual(_systemd.booted(context), 'foo')

    def test_booted_invalid_context(self):
        '''
        Test with invalid context data. The context value must be a dict, so
        this should raise a SaltInvocationError.
        '''
        # Test with invalid context data
        with self.assertRaises(SaltInvocationError):
            _systemd.booted(99999)

    @patch('subprocess.Popen')
    def test_version(self, popen_mock):
        '''
        Test that salt.utils.systemd.booted() returns True when minion is
        systemd-booted.
        '''
        _version = 231
        output = 'systemd {0}\n-SYSVINIT'.format(_version)
        popen_mock.return_value = Mock(
            communicate=lambda *args, **kwargs: (output, None),
            pid=lambda: 12345,
            retcode=0
        )

        # Test without context dict passed
        self.assertEqual(_systemd.version(), _version)
        # Test that context key is set when context dict is passed
        context = {}
        self.assertTrue(_systemd.version(context))
        self.assertEqual(context, {'salt.utils.systemd.version': _version})

    def test_version_return_from_context(self):
        '''
        Test that the context data is returned when present. To ensure we're
        getting data from the context dict, we use a non-integer value to
        differentiate it from the integer return this function normally
        produces.
        '''
        context = {'salt.utils.systemd.version': 'foo'}
        self.assertEqual(_systemd.version(context), 'foo')

    def test_version_invalid_context(self):
        '''
        Test with invalid context data. The context value must be a dict, so
        this should raise a SaltInvocationError.
        '''
        # Test with invalid context data
        with self.assertRaises(SaltInvocationError):
            _systemd.version(99999)

    @patch('subprocess.Popen')
    def test_version_parse_problem(self, popen_mock):
        '''
        Test with invalid context data. The context value must be a dict, so
        this should raise a SaltInvocationError.
        '''
        popen_mock.return_value = Mock(
            communicate=lambda *args, **kwargs: ('invalid', None),
            pid=lambda: 12345,
            retcode=0
        )
        # Test without context dict passed
        self.assertIsNone(_systemd.version())
        # Test that context key is set when context dict is passed. A failure
        # to parse the systemctl output should not set a context key, so it
        # should not be present in the context dict.
        context = {}
        self.assertIsNone(_systemd.version(context))
        self.assertEqual(context, {})

    @patch('subprocess.Popen')
    def test_has_scope_systemd204(self, popen_mock):
        '''
        Scopes are available in systemd>=205. Make sure that this function
        returns the expected boolean. We do three separate unit tests for
        versions 204 through 206 because mock doesn't like us altering the
        return_value in a loop.
        '''
        _expected = False
        _version = 204
        _output = 'systemd {0}\n-SYSVINIT'.format(_version)
        popen_mock.return_value = Mock(
            communicate=lambda *args, **kwargs: (_output, None),
            pid=lambda: 12345,
            retcode=0
        )
        # Ensure that os.stat returns True. os.stat doesn't return a bool
        # normally, but the code is doing a simple truth check on the
        # return data, so it is sufficient enough to mock it as True for
        # these tests.
        with patch('os.stat', side_effect=_booted_effect):
            # Test without context dict passed
            self.assertEqual(_systemd.has_scope(), _expected)
            # Test that context key is set when context dict is passed
            context = {}
            self.assertEqual(_systemd.has_scope(context), _expected)
            self.assertEqual(
                context,
                {'salt.utils.systemd.booted': True,
                 'salt.utils.systemd.version': _version},
            )

    @patch('subprocess.Popen')
    def test_has_scope_systemd205(self, popen_mock):
        '''
        Scopes are available in systemd>=205. Make sure that this function
        returns the expected boolean. We do three separate unit tests for
        versions 204 through 206 because mock doesn't like us altering the
        return_value in a loop.
        '''
        _expected = True
        _version = 205
        _output = 'systemd {0}\n-SYSVINIT'.format(_version)
        popen_mock.return_value = Mock(
            communicate=lambda *args, **kwargs: (_output, None),
            pid=lambda: 12345,
            retcode=0
        )
        # Ensure that os.stat returns True. os.stat doesn't return a bool
        # normally, but the code is doing a simple truth check on the
        # return data, so it is sufficient enough to mock it as True for
        # these tests.
        with patch('os.stat', side_effect=_booted_effect):
            # Test without context dict passed
            self.assertEqual(_systemd.has_scope(), _expected)
            # Test that context key is set when context dict is passed
            context = {}
            self.assertEqual(_systemd.has_scope(context), _expected)
            self.assertEqual(
                context,
                {'salt.utils.systemd.booted': True,
                 'salt.utils.systemd.version': _version},
            )

    @patch('subprocess.Popen')
    def test_has_scope_systemd206(self, popen_mock):
        '''
        Scopes are available in systemd>=205. Make sure that this function
        returns the expected boolean. We do three separate unit tests for
        versions 204 through 206 because mock doesn't like us altering the
        return_value in a loop.
        '''
        _expected = True
        _version = 206
        _output = 'systemd {0}\n-SYSVINIT'.format(_version)
        popen_mock.return_value = Mock(
            communicate=lambda *args, **kwargs: (_output, None),
            pid=lambda: 12345,
            retcode=0
        )
        # Ensure that os.stat returns True. os.stat doesn't return a bool
        # normally, but the code is doing a simple truth check on the
        # return data, so it is sufficient enough to mock it as True for
        # these tests.
        with patch('os.stat', side_effect=_booted_effect):
            # Test without context dict passed
            self.assertEqual(_systemd.has_scope(), _expected)
            # Test that context key is set when context dict is passed
            context = {}
            self.assertEqual(_systemd.has_scope(context), _expected)
            self.assertEqual(
                context,
                {'salt.utils.systemd.booted': True,
                 'salt.utils.systemd.version': _version},
            )

    def test_has_scope_no_systemd(self):
        '''
        Test the case where the system is not systemd-booted. We should not be
        performing a version check in these cases as there is no need.
        '''
        with patch('os.stat', side_effect=_not_booted_effect):
            # Test without context dict passed
            self.assertFalse(_systemd.has_scope())
            # Test that context key is set when context dict is passed.
            # Because we are not systemd-booted, there should be no key in the
            # context dict for the version check, as we shouldn't have
            # performed this check.
            context = {}
            self.assertFalse(_systemd.has_scope(context))
            self.assertEqual(context, {'salt.utils.systemd.booted': False})

    @patch('subprocess.Popen')
    def test_has_scope_version_parse_problem(self, popen_mock):
        '''
        Test the case where the system is systemd-booted, but we failed to
        parse the "systemctl --version" output.
        '''
        popen_mock.return_value = Mock(
            communicate=lambda *args, **kwargs: ('invalid', None),
            pid=lambda: 12345,
            retcode=0
        )
        with patch('os.stat', side_effect=_booted_effect):
            # Test without context dict passed
            self.assertFalse(_systemd.has_scope())
            # Test that context key is set when context dict is passed. A
            # failure to parse the systemctl output should not set a context
            # key, so it should not be present in the context dict.
            context = {}
            self.assertFalse(_systemd.has_scope(context))
            self.assertEqual(context, {'salt.utils.systemd.booted': True})

    def test_has_scope_invalid_context(self):
        '''
        Test with invalid context data. The context value must be a dict, so
        this should raise a SaltInvocationError.
        '''
        # Test with invalid context data
        with self.assertRaises(SaltInvocationError):
            _systemd.has_scope(99999)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(SystemdTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
unit tests for the localfs cache
'''

# Import Python libs
from __future__ import absolute_import
import tempfile

# Import Salt Testing libs
import integration
from salttesting import skipIf, TestCase
from salttesting.helpers import destructiveTest, ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)

ensure_in_syspath('../../')

# Import Salt libs
import salt.payload
import salt.utils
from salt.cache import localfs
from salt.exceptions import SaltCacheError

localfs.__context__ = {}
localfs.__opts__ = {'cachedir': ''}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class LocalFSTest(TestCase):
    '''
    Validate the functions in the localfs cache
    '''

    def _create_tmp_cache_file(self, tmp_dir, serializer):
        '''
        Helper function that creates a temporary cache file using localfs.store. This
        is to used to create DRY unit tests for the localfs cache.
        '''
        with patch.dict(localfs.__opts__, {'cachedir': tmp_dir}):
            with patch.dict(localfs.__context__, {'serial': serializer}):
                localfs.store(bank='bank', key='key', data='payload data', cachedir=tmp_dir)

    # 'store' function tests: 4

    @patch('os.path.isdir', MagicMock(return_value=None))
    @patch('os.makedirs', MagicMock(side_effect=OSError))
    def test_store_no_base_cache_dir(self):
        '''
        Tests that a SaltCacheError is raised when the base directory doesn't exist and
        cannot be created.
        '''
        self.assertRaises(SaltCacheError, localfs.store, bank='', key='', data='', cachedir='')

    @patch('os.path.isdir', MagicMock(return_value=True))
    @patch('tempfile.mkstemp', MagicMock(return_value=(12345, 'foo')))
    def test_store_close_mkstemp_file_handle(self):
        '''
        Tests that the file descriptor that is opened by os.open during the mkstemp call
        in localfs.store is closed before calling salt.utils.fopen on the filename.

        This test mocks the call to mkstemp, but forces an OSError to be raised when the
        close() function is called on a file descriptor that doesn't exist.
        '''
        self.assertRaises(OSError, localfs.store, bank='', key='', data='', cachedir='')

    @patch('os.path.isdir', MagicMock(return_value=True))
    @patch('tempfile.mkstemp', MagicMock(return_value=('one', 'two')))
    @patch('os.close', MagicMock(return_value=None))
    @patch('salt.utils.fopen', MagicMock(side_effect=IOError))
    def test_store_error_writing_cache(self):
        '''
        Tests that a SaltCacheError is raised when there is a problem writing to the
        cache file.
        '''
        self.assertRaises(SaltCacheError, localfs.store, bank='', key='', data='', cachedir='')

    @destructiveTest
    def test_store_success(self):
        '''
        Tests that the store function writes the data to the serializer for storage.
        '''
        # Create a temporary cache dir
        tmp_dir = tempfile.mkdtemp(dir=integration.SYS_TMP_DIR)

        # Use the helper function to create the cache file using localfs.store()
        self._create_tmp_cache_file(tmp_dir, salt.payload.Serial(self))

        # Read in the contents of the key.p file and assert "payload data" was written
        with salt.utils.fopen(tmp_dir + '/bank/key.p') as fh_:
            for line in fh_:
                self.assertIn('payload data', line)

    # 'fetch' function tests: 3

    @patch('os.path.isfile', MagicMock(return_value=False))
    def test_fetch_return_when_cache_file_does_not_exist(self):
        '''
        Tests that the fetch function returns an empty dic when the cache key file
        doesn't exist.
        '''
        self.assertEqual(localfs.fetch(bank='', key='', cachedir=''), {})

    @patch('os.path.isfile', MagicMock(return_value=True))
    @patch('salt.utils.fopen', MagicMock(side_effect=IOError))
    def test_fetch_error_reading_cache(self):
        '''
        Tests that a SaltCacheError is raised when there is a problem reading the cache
        file.
        '''
        self.assertRaises(SaltCacheError, localfs.fetch, bank='', key='', cachedir='')

    @destructiveTest
    def test_fetch_success(self):
        '''
        Tests that the fetch function is able to read the cache file and return its data.
        '''
        # Create a temporary cache dir
        tmp_dir = tempfile.mkdtemp(dir=integration.SYS_TMP_DIR)

        # Create a new serializer object to use in function patches
        serializer = salt.payload.Serial(self)

        # Use the helper function to create the cache file using localfs.store()
        self._create_tmp_cache_file(tmp_dir, serializer)

        # Now fetch the data from the new cache key file
        with patch.dict(localfs.__opts__, {'cachedir': tmp_dir}):
            with patch.dict(localfs.__context__, {'serial': serializer}):
                self.assertIn('payload data', localfs.fetch(bank='bank', key='key', cachedir=tmp_dir))

    # 'updated' function tests: 3

    @patch('os.path.isfile', MagicMock(return_value=False))
    def test_updated_return_when_cache_file_does_not_exist(self):
        '''
        Tests that the updated function returns None when the cache key file doesn't
        exist.
        '''
        self.assertIsNone(localfs.updated(bank='', key='', cachedir=''))

    @patch('os.path.isfile', MagicMock(return_value=True))
    @patch('os.path.getmtime', MagicMock(side_effect=IOError))
    def test_updated_error_when_reading_mtime(self):
        '''
        Tests that a SaltCacheError is raised when there is a problem reading the mtime
        of the cache file.
        '''
        self.assertRaises(SaltCacheError, localfs.updated, bank='', key='', cachedir='')

    @destructiveTest
    def test_updated_success(self):
        '''
        Test that the updated function returns the modification time of the cache file
        '''
        # Create a temporary cache dir
        tmp_dir = tempfile.mkdtemp(dir=integration.SYS_TMP_DIR)

        # Use the helper function to create the cache file using localfs.store()
        self._create_tmp_cache_file(tmp_dir, salt.payload.Serial(self))

        with patch('os.path.join', MagicMock(return_value=tmp_dir + '/bank/key.p')):
            self.assertIsInstance(localfs.updated(bank='bank', key='key', cachedir=tmp_dir), int)

    # 'flush' function tests: 4

    @patch('os.path.isdir', MagicMock(return_value=False))
    def test_flush_key_is_none_and_no_target_dir(self):
        '''
        Tests that the flush function returns False when no key is passed in and the
        target directory doesn't exist.
        '''
        self.assertFalse(localfs.flush(bank='', key=None, cachedir=''))

    @patch('os.path.isfile', MagicMock(return_value=False))
    def test_flush_key_provided_and_no_key_file_false(self):
        '''
        Tests that the flush function returns False when a key file is provided but
        the target key file doesn't exist in the cache bank.
        '''
        self.assertFalse(localfs.flush(bank='', key='key', cachedir=''))

    @patch('os.path.isfile', MagicMock(return_value=True))
    def test_flush_success(self):
        '''
        Tests that the flush function returns True when a key file is provided and
        the target key exists in the cache bank.
        '''
        # Create a temporary cache dir
        tmp_dir = tempfile.mkdtemp(dir=integration.SYS_TMP_DIR)

        # Use the helper function to create the cache file using localfs.store()
        self._create_tmp_cache_file(tmp_dir, salt.payload.Serial(self))

        # Now test the return of the flush function
        with patch.dict(localfs.__opts__, {'cachedir': tmp_dir}):
            self.assertTrue(localfs.flush(bank='bank', key='key', cachedir=tmp_dir))

    @patch('os.path.isfile', MagicMock(return_value=True))
    @patch('os.remove', MagicMock(side_effect=OSError))
    def test_flush_error_raised(self):
        '''
        Tests that a SaltCacheError is raised when there is a problem removing the
        key file from the cache bank
        '''
        self.assertRaises(SaltCacheError, localfs.flush, bank='', key='key', cachedir='/var/cache/salt')

    # 'ls' function tests: 3

    @patch('os.path.isdir', MagicMock(return_value=False))
    def test_ls_no_base_dir(self):
        '''
        Tests that the ls function returns an empty list if the bank directory
        doesn't exist.
        '''
        self.assertEqual(localfs.ls(bank='', cachedir=''), [])

    @patch('os.path.isdir', MagicMock(return_value=True))
    @patch('os.listdir', MagicMock(side_effect=OSError))
    def test_ls_error_raised_no_bank_directory_access(self):
        '''
        Tests that a SaltCacheError is raised when there is a problem accessing the
        cache bank directory.
        '''
        self.assertRaises(SaltCacheError, localfs.ls, bank='', cachedir='')

    @destructiveTest
    def test_ls_success(self):
        '''
        Tests the return of the ls function containing bank entries.
        '''
        # Create a temporary cache dir
        tmp_dir = tempfile.mkdtemp(dir=integration.SYS_TMP_DIR)

        # Use the helper function to create the cache file using localfs.store()
        self._create_tmp_cache_file(tmp_dir, salt.payload.Serial(self))

        # Now test the return of the ls function
        with patch.dict(localfs.__opts__, {'cachedir': tmp_dir}):
            self.assertEqual(localfs.ls(bank='bank', cachedir=tmp_dir), ['key'])

    # 'contains' function tests: 1

    @destructiveTest
    def test_contains(self):
        '''
        Test the return of the contains function when key=None and when a key
        is provided.
        '''
        # Create a temporary cache dir
        tmp_dir = tempfile.mkdtemp(dir=integration.SYS_TMP_DIR)

        # Use the helper function to create the cache file using localfs.store()
        self._create_tmp_cache_file(tmp_dir, salt.payload.Serial(self))

        # Now test the return of the contains function when key=None
        with patch.dict(localfs.__opts__, {'cachedir': tmp_dir}):
            self.assertTrue(localfs.contains(bank='bank', key=None, cachedir=tmp_dir))

        # Now test the return of the contains function when key='key'
        with patch.dict(localfs.__opts__, {'cachedir': tmp_dir}):
            self.assertTrue(localfs.contains(bank='bank', key='key', cachedir=tmp_dir))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(LocalFSTest, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Retrieve EC2 instance data for minions.

The minion id must be the instance-id retrieved from AWS.  As an
option, use_grain can be set to True.  This allows the use of an
instance-id grain instead of the minion-id.  Since this is a potential
security risk, the configuration can be further expanded to include
a list of minions that are trusted to only allow the alternate id
of the instances to specific hosts.  There is no glob matching at
this time.

.. code-block:: yaml

    ext_pillar:
      - ec2_pillar:
          use_grain: True
          minion_ids:
            - trusted-minion-1
            - trusted-minion-2
            - trusted-minion-3

This is a very simple pillar that simply retrieves the instance data
from AWS.  Currently the only portion implemented are EC2 tags, which
returns a list of key/value pairs for all of the EC2 tags assigned to
the instance.

'''

# Import python libs
from __future__ import absolute_import
import re
import logging
from distutils.version import StrictVersion  # pylint: disable=no-name-in-module

# Import AWS Boto libs
try:
    import boto.ec2
    import boto.utils
    import boto.exception
    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False

# Set up logging
log = logging.getLogger(__name__)


def __virtual__():
    '''
    Check for required version of boto and make this pillar available
    depending on outcome.
    '''
    if not HAS_BOTO:
        return False
    boto_version = StrictVersion(boto.__version__)
    required_boto_version = StrictVersion('2.8.0')
    if boto_version < required_boto_version:
        log.error("%s: installed boto version %s < %s, can't retrieve instance data",
                __name__, boto_version, required_boto_version)
        return False
    return True


def _get_instance_info():
    '''
    Helper function to return the instance ID and region of the master where
    this pillar is run.
    '''
    identity = boto.utils.get_instance_identity()['document']
    return (identity['instanceId'], identity['region'])


def ext_pillar(minion_id,
               pillar,  # pylint: disable=W0613
               use_grain=False,
               minion_ids=None):
    '''
    Execute a command and read the output as YAML
    '''

    log.debug("Querying EC2 tags for minion id {0}".format(minion_id))

    # If minion_id is not in the format of an AWS EC2 instance, check to see
    # if there is a grain named 'instance-id' use that.  Because this is a
    # security risk, the master config must contain a use_grain: True option
    # for this external pillar, which defaults to no
    if re.search(r'^i-([0-9a-z]{17}|[0-9a-z]{8})$', minion_id) is None:
        if 'instance-id' not in __grains__:
            log.debug("Minion-id is not in AWS instance-id formation, and there "
                      "is no instance-id grain for minion {0}".format(minion_id))
            return {}
        if not use_grain:
            log.debug("Minion-id is not in AWS instance-id formation, and option "
                      "not set to use instance-id grain, for minion {0}, use_grain "
                      " is {1}".format(
                      minion_id,
                      use_grain))
            return {}
        log.debug("use_grain set to {0}".format(use_grain))
        if minion_ids is not None and minion_id not in minion_ids:
            log.debug("Minion-id is not in AWS instance ID format, and minion_ids "
                      "is set in the ec2_pillar configuration, but minion {0} is "
                      "not in the list of allowed minions {1}".format(minion_id,
                      minion_ids))
            return {}
        if re.search(r'^i-([0-9a-z]{17}|[0-9a-z]{8})$', __grains__['instance-id']) is not None:
            minion_id = __grains__['instance-id']
            log.debug("Minion-id is not in AWS instance ID format, but a grain"
                      " is, so using {0} as the minion ID".format(minion_id))
        else:
            log.debug("Nether minion id nor a grain named instance-id is in "
                      "AWS format, can't query EC2 tags for minion {0}".format(
                      minion_id))
            return {}

    m = boto.utils.get_instance_metadata(timeout=0.1, num_retries=1)
    if len(m.keys()) < 1:
        log.info("%s: not an EC2 instance, skipping", __name__)
        return None

    # Get the Master's instance info, primarily the region
    (instance_id, region) = _get_instance_info()

    try:
        conn = boto.ec2.connect_to_region(region)
    except boto.exception as e:  # pylint: disable=E0712
        log.error("%s: invalid AWS credentials.", __name__)
        return None

    tags = {}
    try:
        _tags = conn.get_all_tags(filters={'resource-type': 'instance',
                'resource-id': minion_id})
        for tag in _tags:
            tags[tag.name] = tag.value
    except IndexError as e:
        log.error("Couldn't retrieve instance information: %s", e)
        return None

    return {'ec2_tags': tags}



# -*- coding: utf-8 -*-
'''
Sudo executor module

@author: Dmitry Kuzmenko <dmitry.kuzmenko@dsr-company.com>
'''
# Import python libs
from __future__ import absolute_import
import json
try:
    from shlex import quote as _cmd_quote  # pylint: disable=E0611
except ImportError:
    from pipes import quote as _cmd_quote

# Import salt libs
from salt.executors import ModuleExecutorBase
import salt.utils
import salt.syspaths

__virtualname__ = 'sudo'


def __virtual__():
    if salt.utils.which('sudo') and __opts__.get('sudo_user'):
        return __virtualname__
    return False


def get(*args, **kwargs):
    return SudoExecutor(*args, **kwargs)


class SudoExecutor(ModuleExecutorBase):
    '''
    Allow for the calling of execution modules via sudo.

    This module is invoked by the minion if the ``sudo_user`` minion config is
    present.

    Example minion config:

    .. code-block:: yaml

        sudo_user: saltdev

    Once this setting is made, any execution module call done by the minion will be
    run under ``sudo -u <sudo_user> salt-call``.  For example, with the above
    minion config,

    .. code-block:: bash

        salt sudo_minion cmd.run 'cat /etc/sudoers'

    is equivalent to

    .. code-block:: bash

        sudo -u saltdev salt-call cmd.run 'cat /etc/sudoers'

    being run on ``sudo_minion``.
    '''

    def __init__(self, opts, data, func, args, kwargs):
        '''
        Constructor
        '''
        super(SudoExecutor, self).__init__()
        self.cmd = ['sudo',
                    '-u', opts.get('sudo_user'),
                    'salt-call',
                    '--out', 'json',
                    '--metadata',
                    '-c', salt.syspaths.CONFIG_DIR,
                    '--',
                    data.get('fun')]
        if data['fun'] == 'state.sls':
            kwargs['concurrent'] = True
        for arg in args:
            self.cmd.append(_cmd_quote(str(arg)))
        for key in kwargs:
            self.cmd.append(_cmd_quote('{0}={1}'.format(key, kwargs[key])))

    def execute(self):
        '''
        Wrap a shell execution out to salt call with sudo

        Example:

        /etc/salt/minion

        .. code-block:: yaml

            sudo_user: saltdev

        .. code-block:: bash

            salt '*' test.ping  # is run as saltdev user
        '''

        cmd_ret = __salt__['cmd.run_all'](self.cmd, use_vt=True, python_shell=False)

        if cmd_ret['retcode'] == 0:
            cmd_meta = json.loads(cmd_ret['stdout'])['local']
            ret = cmd_meta['return']
            __context__['retcode'] = cmd_meta.get('retcode', 0)
        else:
            ret = cmd_ret['stderr']
            __context__['retcode'] = cmd_ret['retcode']

        return ret



# -*- coding: utf-8 -*-
'''
unit tests for the jobs runner
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.runners import jobs
import salt.minion

jobs.__opts__ = {'ext_job_cache': None, 'master_job_cache': 'local_cache'}
jobs.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class JobsTest(TestCase):
    '''
    Validate the jobs runner
    '''
    def test_list_jobs_with_search_target(self):
        '''
        test jobs.list_jobs runner with search_target args
        '''
        mock_jobs_cache = {
            '20160524035503086853': {'Arguments': [],
                                     'Function': 'test.ping',
                                     'StartTime': '2016, May 24 03:55:03.086853',
                                     'Target': 'node-1-1.com',
                                     'Target-type': 'glob',
                                     'User': 'root'},
            '20160524035524895387': {'Arguments': [],
                                     'Function': 'test.ping',
                                     'StartTime': '2016, May 24 03:55:24.895387',
                                     'Target': ['node-1-2.com', 'node-1-1.com'],
                                     'Target-type': 'list',
                                     'User': 'sudo_ubuntu'}
        }

        def return_mock_jobs():
            return mock_jobs_cache

        class MockMasterMinion(object):

            returners = {'local_cache.get_jids': return_mock_jobs}

            def __init__(self, *args, **kwargs):
                pass

        returns = {'all': mock_jobs_cache,
                   'node-1-1.com': mock_jobs_cache,
                   'node-1-2.com': {'20160524035524895387':
                                    mock_jobs_cache['20160524035524895387']},
                   'non-existant': {}}

        with patch.object(salt.minion, 'MasterMinion', MockMasterMinion):
            self.assertEqual(jobs.list_jobs(), returns['all'])

            self.assertEqual(jobs.list_jobs(search_target=['node-1-1*',
                                                           'node-1-2*']),
                             returns['all'])

            self.assertEqual(jobs.list_jobs(search_target='node-1-1.com'),
                             returns['node-1-1.com'])

            self.assertEqual(jobs.list_jobs(search_target='node-1-2.com'),
                             returns['node-1-2.com'])

            self.assertEqual(jobs.list_jobs(search_target='non-existant'),
                             returns['non-existant'])


if __name__ == '__main__':
    from integration import run_tests
    run_tests(JobsTest, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Manage Windows Local Group Policy
=================================

.. versionadded:: 2016.11.0

This state allows configuring local Windows Group Policy

The state can be used to ensure the setting of a single policy or multiple policies in one pass.

Single policies must specify the policy name, the setting, and the policy class (Machine/User/Both)

Example single policy configuration

.. code-block:: yaml

    Ensure Account Lockout Duration:
      lgpo.set:
        - name: Account lockout duration
        - setting: 90
        - policy_class: Machine

.. code-block:: yaml

    Acount lockout duration:
      gpo.set:
        - setting: 120
        - policy_class: Machine

Multiple policy configuration

.. code-block:: yaml

    Company Local Group Policy:
        lgpo.set:
            - computer_policy:
                Deny logon locally: Guest
                Account lockout duration: 120
                Account lockout threshold: 10
                Reset account lockout counter after: 1440
                Enforce password history: 24
                Maximum password age: 60
                Minimum password age: 1
                Minimum password length: 14
                Password must meet complexity requirements: Enabled
                Store passwords using reversible encrytion: Disabled
                Configure Automatic Updates:
                    Configure automatic updating: 4 - Auto download and schedule the intsall
                    Scheduled install day: 7 - Every Saturday
                    Scheduled install time: 17:00
                Specify intranet Microsoft update service location:
                    Set the intranet update service for detecting updates: http://mywsus
                    Set the intranet statistics server: http://mywsus
            - user_policy:
                Do not process the legacy run list: Enabled

.. code-block:: yaml

    server_policy:
      lgpo.set:
        - computer_policy:
            Maximum password age: 60
            Minimum password age: 1
            Minimum password length: 14
            Account lockout duration: 1440
            Account lockout threshold: 10
            Reset account lockout counter after: 1440
            Manage auditing and security log:
              - "BUILTIN\\Administrators"
            Replace a process level token:
              - "NT AUTHORITY\\NETWORK SERVICE"
              - "NT AUTHORITY\\LOCAL SERVICE"
            "Accounts: Guest account status": Disabled
            "Accounts: Rename guest account": Not_4_U
            "Audit: Audit the use of Backup and Restore privilege": Enabled
            "Interactive logon: Do not display last user name": Enabled
            "Network\\DNS Client\\Dynamic update": Disabled
            "System\\Logon\\Do not display the Getting Started welcome screen at logon": Enabled
            "Windows Components\\Remote Desktop Services\\Remote Desktop Session Host\\Connections\\Select RDP transport protocols":
                "Select Transport Type": "Use both UDP and TCP"
            "Windows Components\\Windows Update\\Allow Automatic Updates immediate installation": Enabled
            "Windows Components\\Windows Update\\Allow non-administrators to receive update notifications": Disabled
            "Windows Components\\Windows Update\\Always automatically restart at the scheduled time":
                "The restart timer will give users this much time to save their work (minutes)": 15
            "Windows Components\\Windows Update\\Automatic Updates detection frequency":
                "Check for updates at the following interval (hours)": 1
            "Windows Components\\Windows Update\\Configure Automatic Updates":
                "Configure automatic updating": 4 - Auto download and schedule the install
                "Install during automatic maintenance": False
                "Scheduled install day": 7 - Every Saturday
                "Scheduled install time": "17:00"
            "Windows Components\\Windows Update\\Delay Restart for scheduled installations":
                "Wait the following period before proceeding with a scheduled restart (minutes)": 1
            "Windows Components\\Windows Update\\No auto-restart with logged on users for scheduled automatic updates installations": Disabled
            "Windows Components\\Windows Update\\Re-prompt for restart with scheduled installations":
                "Wait the following period before prompting again with a scheduled restart (minutes)": 30
            "Windows Components\\Windows Update\\Reschedule Automatic Updates scheduled installations": Disabled
            "Windows Components\\Windows Update\\Specify intranet Microsoft update service location":
                "Set the intranet update service for detecting updates": http://mywsus
                "Set the intranet statistics server": http://mywsus
        - cumulative_rights_assignments: True

'''

from __future__ import absolute_import
import logging
import json

log = logging.getLogger(__name__)
__virtualname__ = 'lgpo'
__func_alias__ = {'set_': 'set'}


def __virtual__():
    '''
    load this state if the win_lgpo module exists
    '''
    return __virtualname__ if 'lgpo.set' in __salt__ else False


def set_(name,
         setting=None,
         policy_class=None,
         computer_policy=None,
         user_policy=None,
         cumulative_rights_assignments=True,
         adml_language='en-US'):
    '''
    Ensure the specified policy is set

    name
        the name of a single policy to configure

    setting
        the configuration setting for the single named policy
        if this argument is used the computer_policy/user_policy arguments will be ignored

    policy_class
        the policy class of the single named policy to configure
        this can "machine", "user", or "both"

    computer_policy
        a dict of policyname: value pairs of a set of computer policies to configure
        if this argument is used, the name/setting/policy_class arguments will be ignored

    user_policy
        a dict of policyname: value pairs of a set of user policies to configure
        if this argument is used, the name/setting/policy_class arguments will be ignored

    cumulative_rights_assignments
        determine if any user right assignment policies specified will be cumulative
        or explicit

    adml_language
        the adml language to use for AMDX policy data/display conversions
    '''
    ret = {'name': name,
           'result': True,
           'changes': {},
           'comment': ''}
    policy_classes = ['machine', 'computer', 'user', 'both']
    if not setting and not computer_policy and not user_policy:
        msg = 'At least one of the parameters setting, computer_policy, or user_policy'
        msg = msg + ' must be specified.'
        ret['result'] = False
        ret['comment'] = msg
        return ret
    if setting and not policy_class:
        msg = 'A single policy setting was specified but the policy_class was not specified.'
        ret['result'] = False
        ret['comment'] = msg
        return ret
    if setting and (computer_policy or user_policy):
        msg = 'The setting and computer_policy/user_policy parameters are mutually exclusive.  Please'
        msg = msg + ' specify either a policy name and setting or a computer_policy and/or user_policy'
        msg = msg + ' dict'
        ret['result'] = False
        ret['comment'] = msg
        return ret
    if policy_class and policy_class.lower() not in policy_classes:
        msg = 'The policy_class parameter must be one of the following: {0}'
        ret['result'] = False
        ret['comment'] = msg
        return ret
    if not setting:
        if computer_policy and user_policy:
            policy_class = 'both'
        elif computer_policy:
            policy_class = 'machine'
        elif user_policy:
            policy_class = 'user'
        if computer_policy and not isinstance(computer_policy, dict):
            msg = 'The computer_policy must be specified as a dict.'
            ret['result'] = False
            ret['comment'] = msg
            return ret
        if user_policy and not isinstance(user_policy, dict):
            msg = 'The user_policy must be specified as a dict.'
            ret['result'] = False
            ret['comment'] = msg
            return ret
    else:
        user_policy = {}
        computer_policy = {}
        if policy_class.lower() == 'both':
            user_policy[name] = setting
            computer_policy[name] = setting
        elif policy_class.lower() == 'user':
            user_policy[name] = setting
        elif policy_class.lower() == 'machine' or policy_class.lower() == 'computer':
            computer_policy[name] = setting
    pol_data = {}
    pol_data['user'] = {'output_section': 'User Configuration',
                        'requested_policy': user_policy,
                        'policy_lookup': {}}
    pol_data['machine'] = {'output_section': 'Computer Configuration',
                           'requested_policy': computer_policy,
                           'policy_lookup': {}}

    for p_class, p_data in pol_data.iteritems():
        if p_data['requested_policy']:
            for policy_name, policy_setting in p_data['requested_policy'].iteritems():
                lookup = __salt__['lgpo.get_policy_info'](policy_name,
                                                          p_class,
                                                          adml_language=adml_language)
                if lookup['policy_found']:
                    pol_data[p_class]['policy_lookup'][policy_name] = lookup
                else:
                    ret['comment'] = ' '.join([ret['comment'], lookup['message']])
                    ret['result'] = False
    if not ret['result']:
        return ret

    current_policy = __salt__['lgpo.get'](policy_class=policy_class,
                                          adml_language=adml_language,
                                          hierarchical_return=False)
    log.debug('current policy == {0}'.format(current_policy))

    # compare policies
    policy_changes = []
    for policy_section, policy_data in pol_data.iteritems():
        pol_id = None
        if policy_data and policy_data['output_section'] in current_policy:
            for policy_name, policy_setting in policy_data['requested_policy'].iteritems():
                currently_set = False
                if policy_name in current_policy[policy_data['output_section']]:
                    currently_set = True
                    pol_id = policy_name
                else:
                    for alias in policy_data['policy_lookup'][policy_name]['policy_aliases']:
                        log.debug('checking alias {0}'.format(alias))
                        if alias in current_policy[policy_data['output_section']]:
                            currently_set = True
                            pol_id = alias
                            break
                if currently_set:
                    # compare
                    log.debug('need to compare {0} from current/requested policy'.format(policy_name))
                    changes = False
                    if json.dumps(policy_data['requested_policy'][policy_name], sort_keys=True).lower() != \
                            json.dumps(current_policy[policy_data['output_section']][pol_id], sort_keys=True).lower():
                        if policy_data['policy_lookup'][policy_name]['rights_assignment'] and cumulative_rights_assignments:
                            for user in policy_data['requested_policy'][policy_name]:
                                if user not in current_policy[policy_data['output_section']][pol_id]:
                                    changes = True
                        else:
                            changes = True
                        if changes:
                            log.debug('{0} current policy != requested policy'.format(policy_name))
                            log.debug('we compared {0} to {1}'.format(
                                    json.dumps(policy_data['requested_policy'][policy_name], sort_keys=True).lower(),
                                    json.dumps(current_policy[policy_data['output_section']][pol_id], sort_keys=True).lower()))
                            policy_changes.append(policy_name)
                    else:
                        log.debug('{0} current setting matches the requested setting'.format(policy_name))
                        ret['comment'] = '  '.join(['"{0}" is already set.'.format(policy_name),
                                                    ret['comment']])
                else:
                    policy_changes.append(policy_name)
                    log.debug('policy {0} is not set, we will configure it'.format(policy_name))
    if __opts__['test']:
        if policy_changes:
            ret['result'] = None
            ret['comment'] = 'The following policies are set to change: {0}.'.format(
                    ', '.join(policy_changes))
        else:
            ret['comment'] = 'All specified policies are properly configured'
    else:
        if policy_changes:
            _ret = __salt__['lgpo.set'](computer_policy=computer_policy,
                                        user_policy=user_policy,
                                        cumulative_rights_assignments=cumulative_rights_assignments,
                                        adml_language=adml_language)
            if _ret:
                ret['result'] = _ret
                ret['changes']['old'] = current_policy
                ret['changes']['new'] = __salt__['lgpo.get'](policy_class=policy_class,
                                                             adml_language=adml_language,
                                                             hierarchical_return=False)
            else:
                ret['result'] = False
                ret['comment'] = 'Errors occurred while attempting to configure policies: {0}'.format(_ret)

    return ret



# -*- coding: utf-8 -*-
'''
This runner helps create encrypted passwords that can be included in pillars.

:depends: libnacl, https://github.com/saltstack/libnacl

This is often useful if you wish to store your pillars in source control or
share your pillar data with others that you trust. I don't advise making your pillars public
regardless if they are encrypted or not.

The following configurations can be defined in the master config
so your users can create encrypted passwords using the runner nacl:

.. code-block:: bash

    cat /etc/salt/master.d/nacl.conf
    nacl.config:
        key: 'cKEzd4kXsbeCE7/nLTIqXwnUiD1ulg4NoeeYcCFpd9k='
        keyfile: /root/.nacl

Now with the config in the master you can use the runner nacl like:

.. code-block:: bash

    salt-run nacl.enc 'data'
'''

from __future__ import absolute_import
import base64
import os
import salt.utils
import salt.syspaths


REQ_ERROR = None
try:
    import libnacl.secret
except ImportError as e:
    REQ_ERROR = 'libnacl import error, perhaps missing python libnacl package'

__virtualname__ = 'nacl'


def __virtual__():
    return (REQ_ERROR is None, REQ_ERROR)


def _get_config(**kwargs):
    '''
    Return configuration
    '''
    config = {
        'key': None,
        'keyfile': None,
    }
    config_key = '{0}.config'.format(__virtualname__)
    config.update(__opts__.get(config_key, {}))
    for k in set(config.keys()) & set(kwargs.keys()):
        config[k] = kwargs[k]
    return config


def _get_key(rstrip_newline=True, **kwargs):
    '''
    Return key
    '''
    config = _get_config(**kwargs)
    key = config['key']
    keyfile = config['keyfile']
    if not key and keyfile:
        if not os.path.isfile(keyfile):
            raise Exception('file not found: {0}'.format(keyfile))
        with salt.utils.fopen(keyfile, 'rb') as keyf:
            key = keyf.read()
    if key is None:
        raise Exception('no key found')
    key = str(key)
    if rstrip_newline:
        key = key.rstrip('\n')
    return key


def keygen(keyfile=None):
    '''
    Use libnacl to generate a private key

    CLI Examples:

    .. code-block:: bash

        salt-run nacl.keygen
        salt-run nacl.keygen keyfile=/root/.nacl
        salt-run --out=newline_values_only nacl.keygen > /root/.nacl
    '''
    b = libnacl.secret.SecretBox()
    key = b.sk
    key = base64.b64encode(key)
    if keyfile:
        if os.path.isfile(keyfile):
            raise Exception('file already found: {0}'.format(keyfile))
        with salt.utils.fopen(keyfile, 'w') as keyf:
            keyf.write(key)
            return 'saved: {0}'.format(keyfile)
    return key


def enc(data, **kwargs):
    '''
    Takes a key generated from `nacl.keygen` and encrypt some data.

    CLI Examples:

    .. code-block:: bash

        salt-run nacl.enc datatoenc
        salt-run nacl.enc datatoenc keyfile=/root/.nacl
        salt-run nacl.enc datatoenc key='cKEzd4kXsbeCE7/nLTIqXwnUiD1ulg4NoeeYcCFpd9k='
    '''
    key = _get_key(**kwargs)
    sk = base64.b64decode(key)
    b = libnacl.secret.SecretBox(sk)
    return base64.b64encode(b.encrypt(data))


def dec(data, **kwargs):
    '''
    Takes a key generated from `nacl.keygen` and decrypt some data.

    CLI Examples:

    .. code-block:: bash

        salt-run nacl.dec pEXHQM6cuaF7A=
        salt-run nacl.dec data='pEXHQM6cuaF7A=' keyfile=/root/.nacl
        salt-run nacl.dec data='pEXHQM6cuaF7A=' key='cKEzd4kXsbeCE7/nLTIqXwnUiD1ulg4NoeeYcCFpd9k='
    '''
    key = _get_key(**kwargs)
    sk = base64.b64decode(key)
    b = libnacl.secret.SecretBox(key=sk)
    return b.decrypt(base64.b64decode(data))



# -*- coding: utf-8 -*-
'''
Use a git repository as a Pillar source
---------------------------------------

.. note::
    This external pillar has been rewritten for the :ref:`2015.8.0
    <release-2015-8-0>` release. The old method of configuring this
    external pillar will be maintained for a couple releases, allowing time for
    configurations to be updated to reflect the new usage.

This external pillar allows for a Pillar top file and Pillar SLS files to be
sourced from a git repository.

However, since git_pillar does not have an equivalent to the
:conf_master:`pillar_roots` parameter, configuration is slightly different. A
Pillar top file is required to be in the git repository and must still contain
the relevant environment, like so:

.. code-block:: yaml

    base:
      '*':
        - foo

The branch/tag which maps to that environment must then be specified along with
the repo's URL. Configuration details can be found below.

.. important::
    Each branch/tag used for git_pillar must have its own top file. This is
    different from how the top file works when configuring :ref:`States
    <states-tutorial>`. The reason for this is that each git_pillar branch/tag
    is processed separately from the rest. Therefore, if the ``qa`` branch is
    to be used for git_pillar, it would need to have its own top file, with the
    ``qa`` environment defined within it, like this:

    .. code-block:: yaml

        qa:
          'dev-*':
            - bar

    Additionally, while git_pillar allows for the branch/tag to be overridden
    (see :ref:`here <git-pillar-env-remap>`, or :ref:`here
    <git-pillar-env-remap-legacy>` for Salt releases before 2015.8.0), keep in
    mind that the top file must reference the actual environment name. It is
    common practice to make the environment in a git_pillar top file match the
    branch/tag name, but when remapping, the environment of course no longer
    matches the branch/tag, and the top file needs to be adjusted accordingly.
    When expected Pillar values configured in git_pillar are missing, this is a
    common misconfiguration that may be to blame, and is a good first step in
    troubleshooting.

.. _git-pillar-pre-2015-8-0:

Configuring git_pillar for Salt releases before 2015.8.0
========================================================

.. note::
    This legacy configuration for git_pillar will no longer be supported as of
    the **Oxygen** release of Salt.

For Salt releases earlier than :ref:`2015.8.0 <release-2015-8-0>`,
GitPython is the only supported provider for git_pillar. Individual
repositories can be configured under the :conf_master:`ext_pillar`
configuration parameter like so:

.. code-block:: yaml

    ext_pillar:
      - git: master https://gitserver/git-pillar.git root=subdirectory

The repository is specified in the format ``<branch> <repo_url>``, with an
optional ``root`` parameter (added in the :ref:`2014.7.0
<release-2014-7-0>` release) which allows the pillar SLS files to be
served up from a subdirectory (similar to :conf_master:`gitfs_root` in gitfs).

To use more than one branch from the same repo, multiple lines must be
specified under :conf_master:`ext_pillar`:

.. code-block:: yaml

    ext_pillar:
      - git: master https://gitserver/git-pillar.git
      - git: dev https://gitserver/git-pillar.git

.. _git-pillar-env-remap-legacy:

To remap a specific branch to a specific Pillar environment, use the format
``<branch>:<env>``:

.. code-block:: yaml

    ext_pillar:
      - git: develop:dev https://gitserver/git-pillar.git
      - git: master:prod https://gitserver/git-pillar.git

In this case, the ``develop`` branch would need its own ``top.sls`` with a
``dev`` section in it, like this:

.. code-block:: yaml

    dev:
      '*':
        - bar

The ``master`` branch would need its own ``top.sls`` with a ``prod`` section in
it:

.. code-block:: yaml

    prod:
      '*':
        - bar

If ``__env__`` is specified as the branch name, then git_pillar will first look
at the minion's :conf_minion:`environment` option. If unset, it will fall back
to using branch specified by the master's :conf_master:`gitfs_base`:

.. code-block:: yaml

    ext_pillar:
      - git: __env__ https://gitserver/git-pillar.git root=pillar

The corresponding Pillar top file would look like this:

.. code-block:: yaml

    {{saltenv}}:
      '*':
        - bar

.. note::
    This feature was unintentionally omitted when git_pillar was rewritten for
    the 2015.8.0 release. It was added again in the 2016.3.4 release, but it
    has changed slightly in that release. On Salt masters running 2015.8.0
    through 2016.3.3, this feature can only be accessed using the legacy config
    described above. For 2016.3.4 and later, refer to explanation of the
    ``__env__`` parameter in the below section.

    Versions 2016.3.0 through 2016.3.4 incorrectly check the *master's*
    ``environment`` config option (instead of the minion's) before falling back
    to :conf_master:`gitfs_base`. This has been fixed in the 2016.3.5 and
    2016.11.1 releases (2016.11.0 contains the incorrect behavior).

    Additionally, in releases before 2016.11.0, both ``{{env}}`` and
    ``{{saltenv}}`` could be used as a placeholder for the environment.
    Starting in 2016.11.0, ``{{env}}`` is no longer supported.

.. _git-pillar-2015-8-0-and-later:

Configuring git_pillar for Salt releases 2015.8.0 and later
===========================================================

.. note::
    In version 2015.8.0, the method of configuring git external pillars has
    changed, and now more closely resembles that of the :ref:`Git Fileserver
    Backend <tutorial-gitfs>`. If Salt detects the old configuration schema, it
    will use the pre-2015.8.0 code to compile the external pillar. A warning
    will also be logged.

Beginning with Salt version 2015.8.0, pygit2_ is now supported in addition to
GitPython_ (Dulwich_ will not be supported for the foreseeable future). The
requirements for GitPython_ and pygit2_ are the same as for gitfs, as described
:ref:`here <gitfs-dependencies>`.

.. important::
    git_pillar has its own set of global configuration parameters. While it may
    seem intuitive to use the global gitfs configuration parameters
    (:conf_master:`gitfs_base`, etc.) to manage git_pillar, this will not work.
    The main difference for this is the fact that the different components
    which use Salt's git backend code do not all function identically. For
    instance, in git_pillar it is necessary to specify which branch/tag to be
    used for git_pillar remotes. This is the reverse behavior from gitfs, where
    branches/tags make up your environments.

    See :ref:`here <git_pillar-config-opts>` for documentation on the
    git_pillar configuration options and their usage.

Here is an example git_pillar configuration:

.. code-block:: yaml

    ext_pillar:
      - git:
        # Use 'prod' instead of the branch name 'production' as the environment
        - production https://gitserver/git-pillar.git:
          - env: prod
        # Use 'dev' instead of the branch name 'develop' as the environment
        - develop https://gitserver/git-pillar.git:
          - env: dev
        # No per-remote config parameters (and no trailing colon), 'qa' will
        # be used as the environment
        - qa https://gitserver/git-pillar.git
        # SSH key authentication
        - master git@other-git-server:pillardata-ssh.git:
          # Pillar SLS files will be read from the 'pillar' subdirectory in
          # this repository
          - root: pillar
          - privkey: /path/to/key
          - pubkey: /path/to/key.pub
          - passphrase: CorrectHorseBatteryStaple
        # HTTPS authentication
        - master https://other-git-server/pillardata-https.git:
          - user: git
          - password: CorrectHorseBatteryStaple

The main difference between this and the old way of configuring git_pillar is
that multiple remotes can be configured under one ``git`` section under
:conf_master:`ext_pillar`. More than one ``git`` section can be used, but it is
not necessary. Remotes will be evaluated sequentially.

Per-remote configuration parameters are supported (similar to :ref:`gitfs
<gitfs-per-remote-config>`), and global versions of the git_pillar
configuration parameters can also be set.

.. _git-pillar-env-remap:

To remap a specific branch to a specific Pillar environment, use the ``env``
per-remote parameter:

.. code-block:: yaml

    ext_pillar:
      - git:
        - production https://gitserver/git-pillar.git:
          - env: prod

If ``__env__`` is specified as the branch name, then git_pillar will decide
which branch to use based on the following criteria:

- If the minion has a :conf_minion:`pillarenv` configured, it will use that
  pillar environment. (2016.11.2 and later)
- Otherwise, if the minion has an ``environment`` configured, it will use that
  environment.
- Otherwise, the master's :conf_master:`git_pillar_base` will be used.

.. note::
    The use of :conf_minion:`environment` to choose the pillar environment
    dates from a time before the :conf_minion:`pillarenv` parameter was added.
    In a future release, it will be ignored and either the minion's
    :conf_minion:`pillarenv` or the master's :conf_master:`git_pillar_base`
    will be used.

Here's an example of using ``__env__`` as the git_pillar environment:

.. code-block:: yaml

    ext_pillar:
      - git:
        - __env__ https://gitserver/git-pillar.git:
          - root: pillar

The corresponding Pillar top file would look like this:

.. code-block:: yaml

    {{saltenv}}:
      '*':
        - bar

.. note::
    This feature was unintentionally omitted when git_pillar was rewritten for
    the 2015.8.0 release. It was added again in the 2016.3.4 release, but it
    has changed slightly in that release. The fallback value replaced by
    ``{{env}}`` is :conf_master: is :conf_master:`git_pillar_base`, while the
    legacy config's version of this feature replaces ``{{env}}`` with
    :conf_master:`gitfs_base`.

    On Salt masters running 2015.8.0 through 2016.3.3, this feature can only be
    accessed using the legacy config in the previous section of this page.

    The same issue which affected the behavior of the minion's
    :conf_minion:`environment` config value using the legacy configuration
    syntax (see the documentation in the pre-2015.8.0 section above for the
    legacy support of this feature) also affects the new-style git_pillar
    syntax in version 2016.3.4. This has been corrected in version 2016.3.5 and
    2016.11.1 (2016.11.0 contains the incorrect behavior).

    2016.3.4 incorrectly checks the *master's* ``environment`` config option
    (instead of the minion's) before falling back to the master's
    :conf_master:`git_pillar_base`.

    Additionally, in releases before 2016.11.0, both ``{{env}}`` and
    ``{{saltenv}}`` could be used as a placeholder for the environment.
    Starting in 2016.11.0, ``{{env}}`` is no longer supported.

With the addition of pygit2_ support, git_pillar can now interact with
authenticated remotes. Authentication works just like in gitfs (as outlined in
the :ref:`Git Fileserver Backend Walkthrough <gitfs-authentication>`), only
with the global authenication parameter names prefixed with ``git_pillar``
instead of ``gitfs`` (e.g. :conf_master:`git_pillar_pubkey`,
:conf_master:`git_pillar_privkey`, :conf_master:`git_pillar_passphrase`, etc.).

.. note::
    The ``name`` parameter can be used to further differentiate between two
    remotes with the same URL and branch. When using two remotes with the same
    URL, the ``name`` option is required.

.. _GitPython: https://github.com/gitpython-developers/GitPython
.. _pygit2: https://github.com/libgit2/pygit2
.. _Dulwich: https://www.samba.org/~jelmer/dulwich/
'''
from __future__ import absolute_import

# Import python libs
import copy
import logging
import hashlib
import os

# Import salt libs
import salt.utils
import salt.utils.gitfs
import salt.utils.dictupdate
from salt.exceptions import FileserverConfigError
from salt.pillar import Pillar

# Import third party libs
import salt.ext.six as six
# pylint: disable=import-error
try:
    import git
    HAS_GITPYTHON = True
except ImportError:
    HAS_GITPYTHON = False
# pylint: enable=import-error

PER_REMOTE_OVERRIDES = ('env', 'root', 'ssl_verify')

# Set up logging
log = logging.getLogger(__name__)

# Define the module's virtual name
__virtualname__ = 'git'


def __virtual__():
    '''
    Only load if gitpython is available
    '''
    git_ext_pillars = [x for x in __opts__['ext_pillar'] if 'git' in x]
    if not git_ext_pillars:
        # No git external pillars were configured
        return False

    for ext_pillar in git_ext_pillars:
        if isinstance(ext_pillar['git'], six.string_types):
            # Verification of legacy git pillar configuration
            if not HAS_GITPYTHON:
                log.error(
                    'Git-based ext_pillar is enabled in configuration but '
                    'could not be loaded, is GitPython installed?'
                )
                return False
            if not git.__version__ > '0.3.0':
                return False
            return __virtualname__
        else:
            # Verification of new git pillar configuration
            try:
                salt.utils.gitfs.GitPillar(__opts__)
                # Initialization of the GitPillar object did not fail, so we
                # know we have valid configuration syntax and that a valid
                # provider was detected.
                return __virtualname__
            except FileserverConfigError:
                pass
    return False


def ext_pillar(minion_id, repo, pillar_dirs):
    '''
    Checkout the ext_pillar sources and compile the resulting pillar SLS
    '''
    if isinstance(repo, six.string_types):
        return _legacy_git_pillar(minion_id, repo, pillar_dirs)
    else:
        opts = copy.deepcopy(__opts__)
        opts['pillar_roots'] = {}
        opts['__git_pillar'] = True
        pillar = salt.utils.gitfs.GitPillar(opts)
        pillar.init_remotes(repo, PER_REMOTE_OVERRIDES)
        if __opts__.get('__role') == 'minion':
            # If masterless, fetch the remotes. We'll need to remove this once
            # we make the minion daemon able to run standalone.
            pillar.fetch_remotes()
        pillar.checkout()
        ret = {}
        merge_strategy = __opts__.get(
            'pillar_source_merging_strategy',
            'smart'
        )
        merge_lists = __opts__.get(
            'pillar_merge_lists',
            False
        )
        for pillar_dir, env in six.iteritems(pillar.pillar_dirs):
            log.debug(
                'git_pillar is processing pillar SLS from {0} for pillar '
                'env \'{1}\''.format(pillar_dir, env)
            )
            all_dirs = [d for (d, e) in six.iteritems(pillar.pillar_dirs)
                        if env == e]

            # Ensure that the current pillar_dir is first in the list, so that
            # the pillar top.sls is sourced from the correct location.
            pillar_roots = [pillar_dir]
            pillar_roots.extend([x for x in all_dirs if x != pillar_dir])
            if env == '__env__':
                env = opts.get('pillarenv') \
                    or opts.get('environment') \
                    or opts.get('git_pillar_base')
            opts['pillar_roots'] = {env: pillar_roots}

            local_pillar = Pillar(opts, __grains__, minion_id, env)
            ret = salt.utils.dictupdate.merge(
                ret,
                local_pillar.compile_pillar(ext=False),
                strategy=merge_strategy,
                merge_lists=merge_lists
            )
        return ret


# Legacy git_pillar code
class _LegacyGitPillar(object):
    '''
    Deal with the remote git repository for Pillar
    '''

    def __init__(self, branch, repo_location, opts):
        '''
        Try to initialize the Git repo object
        '''
        self.branch = self.map_branch(branch, opts)
        self.rp_location = repo_location
        self.opts = opts
        self._envs = set()
        self.working_dir = ''
        self.repo = None

        hash_type = getattr(hashlib, opts['hash_type'])
        hash_str = '{0} {1}'.format(self.branch, self.rp_location)
        repo_hash = hash_type(salt.utils.to_bytes(hash_str)).hexdigest()
        rp_ = os.path.join(self.opts['cachedir'], 'pillar_gitfs', repo_hash)

        if not os.path.isdir(rp_):
            os.makedirs(rp_)

        try:
            self.repo = git.Repo.init(rp_)
        except (git.exc.NoSuchPathError,
                git.exc.InvalidGitRepositoryError) as exc:
            log.error('GitPython exception caught while '
                      'initializing the repo: {0}. Maybe '
                      'git is not available.'.format(exc))

        # Git directory we are working on
        # Should be the same as self.repo.working_dir
        self.working_dir = rp_

        if isinstance(self.repo, git.Repo):
            if not self.repo.remotes:
                try:
                    self.repo.create_remote('origin', self.rp_location)
                    # ignore git ssl verification if requested
                    if self.opts.get('pillar_gitfs_ssl_verify', True):
                        self.repo.git.config('http.sslVerify', 'true')
                    else:
                        self.repo.git.config('http.sslVerify', 'false')
                except os.error:
                    # This exception occurs when two processes are
                    # trying to write to the git config at once, go
                    # ahead and pass over it since this is the only
                    # write.
                    # This should place a lock down.
                    pass
            else:
                if self.repo.remotes.origin.url != self.rp_location:
                    self.repo.remotes.origin.config_writer.set(
                        'url', self.rp_location)

    def map_branch(self, branch, opts=None):
        opts = __opts__ if opts is None else opts
        if branch == '__env__':
            branch = opts.get('environment') or 'base'
            if branch == 'base':
                branch = opts.get('gitfs_base') or 'master'
        elif ':' in branch:
            branch = branch.split(':', 1)[0]
        return branch

    def update(self):
        '''
        Ensure you are following the latest changes on the remote

        Return boolean whether it worked
        '''
        try:
            log.debug('Legacy git_pillar: Updating \'%s\'', self.rp_location)
            self.repo.git.fetch()
        except git.exc.GitCommandError as exc:
            log.error('Unable to fetch the latest changes from remote '
                      '{0}: {1}'.format(self.rp_location, exc))
            return False

        try:
            checkout_ref = 'origin/{0}'.format(self.branch)
            log.debug('Legacy git_pillar: Checking out %s for \'%s\'',
                      checkout_ref, self.rp_location)
            self.repo.git.checkout(checkout_ref)
        except git.exc.GitCommandError as exc:
            log.error(
                'Legacy git_pillar: Failed to checkout %s for \'%s\': %s',
                checkout_ref, self.rp_location, exc
            )
            return False

        return True

    def envs(self):
        '''
        Return a list of refs that can be used as environments
        '''
        if isinstance(self.repo, git.Repo):
            remote = self.repo.remote()
            for ref in self.repo.refs:
                parted = ref.name.partition('/')
                short = parted[2] if parted[2] else parted[0]
                if isinstance(ref, git.Head):
                    if short == 'master':
                        short = 'base'
                    if ref not in remote.stale_refs:
                        self._envs.add(short)
                elif isinstance(ref, git.Tag):
                    self._envs.add(short)

        return list(self._envs)


def _legacy_git_pillar(minion_id, repo_string, pillar_dirs):
    '''
    Support pre-Beryllium config schema
    '''
    salt.utils.warn_until(
        'Oxygen',
        'The git ext_pillar configuration is deprecated. Please refer to the '
        'documentation at '
        'https://docs.saltstack.com/en/latest/ref/pillar/all/salt.pillar.git_pillar.html '
        'for more information. This configuration will no longer be supported '
        'as of the Oxygen release of Salt.'
    )
    if pillar_dirs is None:
        return
    # split the branch, repo name and optional extra (key=val) parameters.
    options = repo_string.strip().split()
    branch_env = options[0]
    repo_location = options[1]
    root = ''

    for extraopt in options[2:]:
        # Support multiple key=val attributes as custom parameters.
        DELIM = '='
        if DELIM not in extraopt:
            log.error(
                'Legacy git_pillar: Incorrectly formatted extra parameter '
                '\'%s\' within \'%s\' missing \'%s\')',
                extraopt, repo_string, DELIM
            )
        key, val = _extract_key_val(extraopt, DELIM)
        if key == 'root':
            root = val
        else:
            log.error(
                'Legacy git_pillar: Unrecognized extra parameter \'%s\' '
                'in \'%s\'',
                key, repo_string
            )

    # environment is "different" from the branch
    cfg_branch, _, environment = branch_env.partition(':')

    gitpil = _LegacyGitPillar(cfg_branch, repo_location, __opts__)
    branch = gitpil.branch

    if environment == '':
        if branch == 'master':
            environment = 'base'
        else:
            environment = branch

    # normpath is needed to remove appended '/' if root is empty string.
    pillar_dir = os.path.normpath(os.path.join(gitpil.working_dir, root))
    log.debug(
        'Legacy git_pillar: pillar_dir for \'%s\' is \'%s\'',
        repo_string, pillar_dir
    )
    log.debug(
        'Legacy git_pillar: branch for \'%s\' is \'%s\'',
        repo_string, branch
    )

    pillar_dirs.setdefault(pillar_dir, {})

    if cfg_branch == '__env__' and branch not in ['master', 'base']:
        gitpil.update()
    elif pillar_dirs[pillar_dir].get(branch, False):
        log.debug(
            'Already processed pillar_dir \'%s\' for \'%s\'',
            pillar_dir, repo_string
        )
        return {}  # we've already seen this combo

    pillar_dirs[pillar_dir].setdefault(branch, True)

    # Don't recurse forever-- the Pillar object will re-call the ext_pillar
    # function
    if __opts__['pillar_roots'].get(branch, []) == [pillar_dir]:
        return {}

    opts = copy.deepcopy(__opts__)

    opts['pillar_roots'][environment] = [pillar_dir]
    opts['__git_pillar'] = True

    pil = Pillar(opts, __grains__, minion_id, branch)

    return pil.compile_pillar(ext=False)


def _update(branch, repo_location):
    '''
    Ensure you are following the latest changes on the remote

    return boolean whether it worked
    '''
    gitpil = _LegacyGitPillar(branch, repo_location, __opts__)

    return gitpil.update()


def _envs(branch, repo_location):
    '''
    Return a list of refs that can be used as environments
    '''
    gitpil = _LegacyGitPillar(branch, repo_location, __opts__)

    return gitpil.envs()


def _extract_key_val(kv, delimiter='='):
    '''Extract key and value from key=val string.

    Example:
    >>> _extract_key_val('foo=bar')
    ('foo', 'bar')
    '''
    pieces = kv.split(delimiter)
    key = pieces[0]
    val = delimiter.join(pieces[1:])
    return key, val



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    create_autospec,
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import etcd_mod
from salt.utils import etcd_util

# Globals
etcd_mod.__opts__ = {}
etcd_mod.__utils__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class EtcdModTestCase(TestCase):
    '''
    Test cases for salt.modules.etcd_mod
    '''

    def setUp(self):
        self.instance = create_autospec(etcd_util.EtcdClient)
        self.EtcdClientMock = MagicMock()
        self.EtcdClientMock.return_value = self.instance

    def tearDown(self):
        self.instance = None
        self.EtcdClientMock = None

    # 'get_' function tests: 1

    def test_get(self):
        '''
        Test if it get a value from etcd, by direct path
        '''
        with patch.dict(etcd_mod.__utils__, {'etcd_util.get_conn': self.EtcdClientMock}):
            self.instance.get.return_value = 'stack'
            self.assertEqual(etcd_mod.get_('salt'), 'stack')
            self.instance.get.assert_called_with('salt', recurse=False)

            self.instance.tree.return_value = {}
            self.assertEqual(etcd_mod.get_('salt', recurse=True), {})
            self.instance.tree.assert_called_with('salt')

            self.instance.get.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.get_, 'err')

    # 'set_' function tests: 1

    def test_set(self):
        '''
        Test if it set a key in etcd, by direct path
        '''
        with patch.dict(etcd_mod.__utils__, {'etcd_util.get_conn': self.EtcdClientMock}):
            self.instance.set.return_value = 'stack'
            self.assertEqual(etcd_mod.set_('salt', 'stack'), 'stack')
            self.instance.set.assert_called_with('salt', 'stack', directory=False, ttl=None)

            self.instance.set.return_value = True
            self.assertEqual(etcd_mod.set_('salt', '', directory=True), True)
            self.instance.set.assert_called_with('salt', '', directory=True, ttl=None)

            self.assertEqual(etcd_mod.set_('salt', '', directory=True, ttl=5), True)
            self.instance.set.assert_called_with('salt', '', directory=True, ttl=5)

            self.assertEqual(etcd_mod.set_('salt', '', None, 10, True), True)
            self.instance.set.assert_called_with('salt', '', directory=True, ttl=10)

            self.instance.set.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.set_, 'err', 'stack')

    # 'update' function tests: 1

    def test_update(self):
        '''
        Test if can set multiple keys in etcd
        '''
        with patch.dict(etcd_mod.__utils__, {'etcd_util.get_conn': self.EtcdClientMock}):
            args = {
                'x': {
                    'y': {
                        'a': '1',
                        'b': '2',
                    }
                },
                'z': '4',
                'd': {},
            }

            result = {
                '/some/path/x/y/a': '1',
                '/some/path/x/y/b': '2',
                '/some/path/z': '4',
                '/some/path/d': {},
            }
            self.instance.update.return_value = result
            self.assertDictEqual(etcd_mod.update(args, path='/some/path'), result)
            self.instance.update.assert_called_with(args, '/some/path')
            self.assertDictEqual(etcd_mod.update(args), result)
            self.instance.update.assert_called_with(args, '')

    # 'ls_' function tests: 1

    def test_ls(self):
        '''
        Test if it return all keys and dirs inside a specific path
        '''
        with patch.dict(etcd_mod.__utils__, {'etcd_util.get_conn': self.EtcdClientMock}):
            self.instance.ls.return_value = {'/some-dir': {}}
            self.assertDictEqual(etcd_mod.ls_('/some-dir'), {'/some-dir': {}})
            self.instance.ls.assert_called_with('/some-dir')

            self.instance.ls.return_value = {'/': {}}
            self.assertDictEqual(etcd_mod.ls_(), {'/': {}})
            self.instance.ls.assert_called_with('/')

            self.instance.ls.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.ls_, 'err')

    # 'rm_' function tests: 1

    def test_rm(self):
        '''
        Test if it delete a key from etcd
        '''
        with patch.dict(etcd_mod.__utils__, {'etcd_util.get_conn': self.EtcdClientMock}):
            self.instance.rm.return_value = False
            self.assertFalse(etcd_mod.rm_('dir'))
            self.instance.rm.assert_called_with('dir', recurse=False)

            self.instance.rm.return_value = True
            self.assertTrue(etcd_mod.rm_('dir', recurse=True))
            self.instance.rm.assert_called_with('dir', recurse=True)

            self.instance.rm.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.rm_, 'err')

    # 'tree' function tests: 1

    def test_tree(self):
        '''
        Test if it recurses through etcd and return all values
        '''
        with patch.dict(etcd_mod.__utils__, {'etcd_util.get_conn': self.EtcdClientMock}):
            self.instance.tree.return_value = {}
            self.assertDictEqual(etcd_mod.tree('/some-dir'), {})
            self.instance.tree.assert_called_with('/some-dir')

            self.assertDictEqual(etcd_mod.tree(), {})
            self.instance.tree.assert_called_with('/')

            self.instance.tree.side_effect = Exception
            self.assertRaises(Exception, etcd_mod.tree, 'err')

    # 'watch' function tests: 1

    def test_watch(self):
        '''
        Test if watch returns the right tuples
        '''
        with patch.dict(etcd_mod.__utils__, {'etcd_util.get_conn': self.EtcdClientMock}):
            self.instance.watch.return_value = {
                'value': 'stack',
                'changed': True,
                'dir': False,
                'mIndex': 1,
                'key': '/salt'
            }
            self.assertEqual(etcd_mod.watch('/salt'), self.instance.watch.return_value)
            self.instance.watch.assert_called_with('/salt', recurse=False, timeout=0, index=None)

            self.instance.watch.return_value['dir'] = True
            self.assertEqual(etcd_mod.watch('/some-dir', recurse=True, timeout=5, index=10),
                             self.instance.watch.return_value)
            self.instance.watch.assert_called_with('/some-dir', recurse=True, timeout=5, index=10)

            self.assertEqual(etcd_mod.watch('/some-dir', True, None, 5, 10),
                             self.instance.watch.return_value)
            self.instance.watch.assert_called_with('/some-dir', recurse=True, timeout=5, index=10)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(EtcdModTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Support for the Amazon Simple Queue Service.
'''

# Import Python libs
from __future__ import absolute_import
import logging
import json

# Import salt libs
import salt.utils
import salt.ext.six as six

log = logging.getLogger(__name__)

_OUTPUT = '--output json'


def __virtual__():
    if salt.utils.which('aws'):
        # awscli is installed, load the module
        return True
    return (False, 'The module aws_sqs could not be loaded: aws command not found')


def _region(region):
    '''
    Return the region argument.
    '''
    return ' --region {r}'.format(r=region)


def _run_aws(cmd, region, opts, user, **kwargs):
    '''
    Runs the given command against AWS.
    cmd
        Command to run
    region
        Region to execute cmd in
    opts
        Pass in from salt
    user
        Pass in from salt
    kwargs
        Key-value arguments to pass to the command
    '''
    # These args need a specific key value that aren't
    # valid python parameter keys
    receipthandle = kwargs.pop('receipthandle', None)
    if receipthandle:
        kwargs['receipt-handle'] = receipthandle
    num = kwargs.pop('num', None)
    if num:
        kwargs['max-number-of-messages'] = num

    _formatted_args = [
        '--{0} "{1}"'.format(k, v) for k, v in six.iteritems(kwargs)]

    cmd = 'aws sqs {cmd} {args} {region} {out}'.format(
        cmd=cmd,
        args=' '.join(_formatted_args),
        region=_region(region),
        out=_OUTPUT)

    rtn = __salt__['cmd.run'](cmd, runas=user, python_shell=False)

    return json.loads(rtn) if rtn else ''


def receive_message(queue, region, num=1, opts=None, user=None):
    '''
    Receive one or more messages from a queue in a region

    queue
        The name of the queue to receive messages from

    region
        Region where SQS queues exists

    num : 1
        The max number of messages to receive

    opts : None
        Any additional options to add to the command line

    user : None
        Run as a user other than what the minion runs as

    CLI Example:

    .. code-block:: bash

        salt '*' aws_sqs.receive_message <sqs queue> <region>
        salt '*' aws_sqs.receive_message <sqs queue> <region> num=10

    .. versionadded:: 2014.7.0

    '''
    ret = {
            'Messages': None,
          }
    queues = list_queues(region, opts, user)
    url_map = _parse_queue_list(queues)
    if queue not in url_map:
        log.info('"{0}" queue does not exist.'.format(queue))
        return ret

    out = _run_aws('receive-message', region, opts, user, queue=url_map[queue],
                   num=num)
    ret['Messages'] = out['Messages']
    return ret


def delete_message(queue, region, receipthandle, opts=None, user=None):
    '''
    Delete one or more messages from a queue in a region

    queue
        The name of the queue to delete messages from

    region
        Region where SQS queues exists

    receipthandle
        The ReceiptHandle of the message to delete. The ReceiptHandle
        is obtained in the return from receive_message

    opts : None
        Any additional options to add to the command line

    user : None
        Run as a user other than what the minion runs as

    CLI Example:

    .. code-block:: bash

        salt '*' aws_sqs.delete_message <sqs queue> <region> receipthandle='<sqs ReceiptHandle>'

    .. versionadded:: 2014.7.0

    '''
    queues = list_queues(region, opts, user)
    url_map = _parse_queue_list(queues)
    if queue not in url_map:
        log.info('"{0}" queue does not exist.'.format(queue))
        return False

    out = _run_aws('delete-message', region, opts, user,
                   receipthandle=receipthandle, queue=url_map[queue],)
    return True


def list_queues(region, opts=None, user=None):
    '''
    List the queues in the selected region.

    region
        Region to list SQS queues for

    opts : None
        Any additional options to add to the command line

    user : None
        Run hg as a user other than what the minion runs as

    CLI Example:

        salt '*' aws_sqs.list_queues <region>

    '''
    out = _run_aws('list-queues', region, opts, user)

    ret = {
        'retcode': 0,
        'stdout': out['QueueUrls'],
    }
    return ret


def create_queue(name, region, opts=None, user=None):
    '''
    Creates a queue with the correct name.

    name
        Name of the SQS queue to create

    region
        Region to create the SQS queue in

    opts : None
        Any additional options to add to the command line

    user : None
        Run hg as a user other than what the minion runs as

    CLI Example:

        salt '*' aws_sqs.create_queue <sqs queue> <region>

    '''

    create = {'queue-name': name}
    out = _run_aws(
        'create-queue', region=region, opts=opts,
        user=user, **create)

    ret = {
        'retcode': 0,
        'stdout': out['QueueUrl'],
        'stderr': '',
    }
    return ret


def delete_queue(name, region, opts=None, user=None):
    '''
    Deletes a queue in the region.

    name
        Name of the SQS queue to deletes
    region
        Name of the region to delete the queue from

    opts : None
        Any additional options to add to the command line

    user : None
        Run hg as a user other than what the minion runs as

    CLI Example:

        salt '*' aws_sqs.delete_queue <sqs queue> <region>

    '''
    queues = list_queues(region, opts, user)
    url_map = _parse_queue_list(queues)

    logger = logging.getLogger(__name__)
    logger.debug('map ' + six.text_type(url_map))
    if name in url_map:
        delete = {'queue-url': url_map[name]}

        rtn = _run_aws(
            'delete-queue',
            region=region,
            opts=opts,
            user=user,
            **delete)
        success = True
        err = ''
        out = '{0} deleted'.format(name)

    else:
        out = ''
        err = "Delete failed"
        success = False

    ret = {
        'retcode': 0 if success else 1,
        'stdout': out,
        'stderr': err,
    }
    return ret


def queue_exists(name, region, opts=None, user=None):
    '''
    Returns True or False on whether the queue exists in the region

    name
        Name of the SQS queue to search for

    region
        Name of the region to search for the queue in

    opts : None
        Any additional options to add to the command line

    user : None
        Run hg as a user other than what the minion runs as

    CLI Example:

        salt '*' aws_sqs.queue_exists <sqs queue> <region>

    '''
    output = list_queues(region, opts, user)

    return name in _parse_queue_list(output)


def _parse_queue_list(list_output):
    '''
    Parse the queue to get a dict of name -> URL
    '''
    queues = dict((q.split('/')[-1], q) for q in list_output['stdout'])
    return queues



# -*- coding: utf-8 -*-
'''
A lightweight version of tests.integration for testing of unit tests

This test class will not import the salt minion, runner and config modules.
'''
from __future__ import absolute_import
from salttesting.case import TestCase
from salttesting.parser import SaltTestcaseParser

__all__ = ['run_tests', 'ModuleTestCase']


def run_tests(*test_cases, **kwargs):
    '''
    Run unit tests for the chosen test cases.

    :param test_cases: The list of test cases to execute
    :type  test_cases: ``list`` of :class:`TestCase`
    '''
    parser = SaltTestcaseParser()
    parser.parse_args()
    for case in test_cases:
        if parser.run_testcase(case) is False:
            parser.finalize(1)
    parser.finalize(0)


def hasDependency(module, fake_module=None):
    '''
    Use this function in your test class setUp to
    mock modules into your namespace

    :param module: The module name
    :type  module: ``str``

    :param fake_module: The module to inject into sys.modules
        if not provided, a mock will be injected. If ``False``
        is passed explicitly, the module injection does not
        occur.

    :type  fake_module: ``object``

    ..
        hasDependency('super_module')
    '''
    import mock
    import sys
    if fake_module is None:
        fake_module = mock.MagicMock()

    if fake_module:
        sys.modules[module] = fake_module


class MockLoader(object):
    '''
    The default replacement for __salt__'s loader
    class.
    '''
    def set_result(self, module, key, func):
        if module.__salt__ is None:
            module.__salt__ = {}
        module.__salt__[key] = func


class ModuleTestCase(TestCase):
    '''
    A base class for test cases of execution modules

    ..
        class MyModuleTestCase(ModuleTestCase):
            def setUp(self):
                self.setup_loader()
    '''
    # Set this class-level attribute to change
    # the loader behavior
    loaderCls = MockLoader

    def setup_loader(self):
        '''
        Instantiate a loader to your test case
        '''
        self.loader = self.loaderCls()



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import
import textwrap

# Import Salt Libs
from salt.modules import parallels
from salt.exceptions import SaltInvocationError

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import MagicMock, patch, NO_MOCK, NO_MOCK_REASON
from salttesting.helpers import ensure_in_syspath

# Import third party libs
import salt.ext.six as six

ensure_in_syspath('../../')
parallels.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ParallelsTestCase(TestCase):
    '''
    Test parallels desktop execution module functions
    '''
    def test___virtual__(self):
        '''
        Test parallels.__virtual__
        '''
        mock_true = MagicMock(return_value=True)
        mock_false = MagicMock(return_value=False)

        # Validate false return
        with patch('salt.utils.which', mock_false):
            ret = parallels.__virtual__()
            self.assertTrue(isinstance(ret, tuple))
            self.assertEqual(len(ret), 2)
            self.assertFalse(ret[0])
            self.assertTrue(isinstance(ret[1], six.string_types))

        # Validate true return
        with patch('salt.utils.which', mock_true):
            ret = parallels.__virtual__()
            self.assertTrue(ret)
            self.assertEqual(ret, 'parallels')

    def test__normalize_args(self):
        '''
        Test parallels._normalize_args
        '''
        def _validate_ret(ret):
            '''
            Assert that the returned data is a list of strings
            '''
            self.assertTrue(isinstance(ret, list))
            for arg in ret:
                self.assertTrue(isinstance(arg, six.string_types))

        # Validate string arguments
        str_args = 'electrolytes --aqueous --anion hydroxide --cation=ammonium free radicals -- hydrogen'
        _validate_ret(parallels._normalize_args(str_args))

        # Validate list arguments
        list_args = ' '.join(str_args)
        _validate_ret(parallels._normalize_args(list_args))

        # Validate tuple arguments
        tuple_args = tuple(list_args)
        _validate_ret(parallels._normalize_args(tuple_args))

        # Validate dictionary arguments
        other_args = {'anion': 'hydroxide', 'cation': 'ammonium'}
        _validate_ret(parallels._normalize_args(other_args))

    def test__find_guids(self):
        '''
        Test parallels._find_guids
        '''
        guid_str = textwrap.dedent('''
            PARENT_SNAPSHOT_ID                      SNAPSHOT_ID
                                                    {a5b8999f-5d95-4aff-82de-e515b0101b66}
            {a5b8999f-5d95-4aff-82de-e515b0101b66} *{a7345be5-ab66-478c-946e-a6c2caf14909}
        ''')
        guids = ['a5b8999f-5d95-4aff-82de-e515b0101b66',
                     'a7345be5-ab66-478c-946e-a6c2caf14909']

        self.assertEqual(parallels._find_guids(guid_str), guids)

    def test_prlsrvctl(self):
        '''
        Test parallels.prlsrvctl
        '''
        runas = 'macdev'

        # Validate 'prlsrvctl info'
        info_cmd = ['prlsrvctl', 'info']
        info_fcn = MagicMock()
        with patch.dict(parallels.__salt__, {'cmd.run': info_fcn}):
            parallels.prlsrvctl('info', runas=runas)
            info_fcn.assert_called_once_with(info_cmd, runas=runas)

        # Validate 'prlsrvctl usb list'
        usb_cmd = ['prlsrvctl', 'usb', 'list']
        usb_fcn = MagicMock()
        with patch.dict(parallels.__salt__, {'cmd.run': usb_fcn}):
            parallels.prlsrvctl('usb', 'list', runas=runas)
            usb_fcn.assert_called_once_with(usb_cmd, runas=runas)

        # Validate 'prlsrvctl set "--mem-limit auto"'
        set_cmd = ['prlsrvctl', 'set', '--mem-limit', 'auto']
        set_fcn = MagicMock()
        with patch.dict(parallels.__salt__, {'cmd.run': set_fcn}):
            parallels.prlsrvctl('set', '--mem-limit auto', runas=runas)
            set_fcn.assert_called_once_with(set_cmd, runas=runas)

    def test_prlctl(self):
        '''
        Test parallels.prlctl
        '''
        runas = 'macdev'

        # Validate 'prlctl user list'
        user_cmd = ['prlctl', 'user', 'list']
        user_fcn = MagicMock()
        with patch.dict(parallels.__salt__, {'cmd.run': user_fcn}):
            parallels.prlctl('user', 'list', runas=runas)
            user_fcn.assert_called_once_with(user_cmd, runas=runas)

        # Validate 'prlctl exec "macvm uname"'
        exec_cmd = ['prlctl', 'exec', 'macvm', 'uname']
        exec_fcn = MagicMock()
        with patch.dict(parallels.__salt__, {'cmd.run': exec_fcn}):
            parallels.prlctl('exec', 'macvm uname', runas=runas)
            exec_fcn.assert_called_once_with(exec_cmd, runas=runas)

        # Validate 'prlctl capture "macvm --file macvm.display.png"'
        cap_cmd = ['prlctl', 'capture', 'macvm', '--file', 'macvm.display.png']
        cap_fcn = MagicMock()
        with patch.dict(parallels.__salt__, {'cmd.run': cap_fcn}):
            parallels.prlctl('capture', 'macvm --file macvm.display.png', runas=runas)
            cap_fcn.assert_called_once_with(cap_cmd, runas=runas)

    def test_list_vms(self):
        '''
        Test parallels.list_vms
        '''
        runas = 'macdev'

        # Validate a simple list
        mock_plain = MagicMock()
        with patch.object(parallels, 'prlctl', mock_plain):
            parallels.list_vms(runas=runas)
            mock_plain.assert_called_once_with('list', [], runas=runas)

        # Validate listing a single VM
        mock_name = MagicMock()
        with patch.object(parallels, 'prlctl', mock_name):
            parallels.list_vms(name='macvm', runas=runas)
            mock_name.assert_called_once_with('list',
                                              ['macvm'],
                                              runas=runas)

        # Validate listing templates
        mock_templ = MagicMock()
        with patch.object(parallels, 'prlctl', mock_templ):
            parallels.list_vms(template=True, runas=runas)
            mock_templ.assert_called_once_with('list',
                                               ['--template'],
                                               runas=runas)

        # Validate listing extra info
        mock_info = MagicMock()
        with patch.object(parallels, 'prlctl', mock_info):
            parallels.list_vms(info=True, runas=runas)
            mock_info.assert_called_once_with('list',
                                              ['--info'],
                                              runas=runas)

        # Validate listing with extra options
        mock_complex = MagicMock()
        with patch.object(parallels, 'prlctl', mock_complex):
            parallels.list_vms(args=' -o uuid,status', all=True, runas=runas)
            mock_complex.assert_called_once_with('list',
                                                 ['-o', 'uuid,status', '--all'],
                                                 runas=runas)

    def test_clone(self):
        '''
        Test parallels.clone
        '''
        name = 'macvm'
        runas = 'macdev'

        # Validate clone
        mock_clone = MagicMock()
        with patch.object(parallels, 'prlctl', mock_clone):
            parallels.clone(name, 'macvm_new', runas=runas)
            mock_clone.assert_called_once_with('clone',
                                               [name, '--name', 'macvm_new'],
                                               runas=runas)

        # Validate linked clone
        mock_linked = MagicMock()
        with patch.object(parallels, 'prlctl', mock_linked):
            parallels.clone(name, 'macvm_link', linked=True, runas=runas)
            mock_linked.assert_called_once_with('clone',
                                                [name, '--name', 'macvm_link', '--linked'],
                                                runas=runas)

        # Validate template clone
        mock_template = MagicMock()
        with patch.object(parallels, 'prlctl', mock_template):
            parallels.clone(name, 'macvm_templ', template=True, runas=runas)
            mock_template.assert_called_once_with('clone',
                                                  [name, '--name', 'macvm_templ', '--template'],
                                                  runas=runas)

    def test_delete(self):
        '''
        Test parallels.delete
        '''
        name = 'macvm'
        runas = 'macdev'

        # Validate delete
        mock_delete = MagicMock()
        with patch.object(parallels, 'prlctl', mock_delete):
            parallels.delete(name, runas=runas)
            mock_delete.assert_called_once_with('delete', name, runas=runas)

    def test_exists(self):
        '''
        Test parallels.exists
        '''
        name = 'macvm'
        runas = 'macdev'

        # Validate exists
        mock_list = MagicMock(return_value='Name: {0}\nState: running'.format(name))
        with patch.object(parallels, 'list_vms', mock_list):
            self.assertTrue(parallels.exists(name, runas=runas))

        # Validate not exists
        mock_list = MagicMock(return_value='Name: {0}\nState: running'.format(name))
        with patch.object(parallels, 'list_vms', mock_list):
            self.assertFalse(parallels.exists('winvm', runas=runas))

    def test_start(self):
        '''
        Test parallels.start
        '''
        name = 'macvm'
        runas = 'macdev'

        mock_start = MagicMock()
        with patch.object(parallels, 'prlctl', mock_start):
            parallels.start(name, runas=runas)
            mock_start.assert_called_once_with('start', name, runas=runas)

    def test_stop(self):
        '''
        Test parallels.stop
        '''
        name = 'macvm'
        runas = 'macdev'

        # Validate stop
        mock_stop = MagicMock()
        with patch.object(parallels, 'prlctl', mock_stop):
            parallels.stop(name, runas=runas)
            mock_stop.assert_called_once_with('stop', [name], runas=runas)

        # Validate immediate stop
        mock_kill = MagicMock()
        with patch.object(parallels, 'prlctl', mock_kill):
            parallels.stop(name, kill=True, runas=runas)
            mock_kill.assert_called_once_with('stop', [name, '--kill'], runas=runas)

    def test_restart(self):
        '''
        Test parallels.restart
        '''
        name = 'macvm'
        runas = 'macdev'

        mock_start = MagicMock()
        with patch.object(parallels, 'prlctl', mock_start):
            parallels.restart(name, runas=runas)
            mock_start.assert_called_once_with('restart', name, runas=runas)

    def test_reset(self):
        '''
        Test parallels.reset
        '''
        name = 'macvm'
        runas = 'macdev'

        mock_start = MagicMock()
        with patch.object(parallels, 'prlctl', mock_start):
            parallels.reset(name, runas=runas)
            mock_start.assert_called_once_with('reset', name, runas=runas)

    def test_status(self):
        '''
        Test parallels.status
        '''
        name = 'macvm'
        runas = 'macdev'

        mock_start = MagicMock()
        with patch.object(parallels, 'prlctl', mock_start):
            parallels.status(name, runas=runas)
            mock_start.assert_called_once_with('status', name, runas=runas)

    def test_exec_(self):
        '''
        Test parallels.exec_
        '''
        name = 'macvm'
        runas = 'macdev'

        mock_start = MagicMock()
        with patch.object(parallels, 'prlctl', mock_start):
            parallels.exec_(name, 'find /etc/paths.d', runas=runas)
            mock_start.assert_called_once_with('exec',
                                               [name, 'find', '/etc/paths.d'],
                                               runas=runas)

    def test_snapshot_id_to_name(self):
        '''
        Test parallels.snapshot_id_to_name
        '''
        name = 'macvm'
        snap_id = 'a5b8999f-5d95-4aff-82de-e515b0101b66'

        # Invalid GUID raises error
        self.assertRaises(SaltInvocationError,
                          parallels.snapshot_id_to_name,
                          name,
                          '{8-4-4-4-12}')

        # Empty return from prlctl raises error (name/snap_id mismatch?)
        mock_no_data = MagicMock(return_value='')
        with patch.object(parallels, 'prlctl', mock_no_data):
            self.assertRaises(SaltInvocationError,
                              parallels.snapshot_id_to_name,
                              name,
                              snap_id)

        # Data returned from prlctl is invalid YAML
        mock_invalid_data = MagicMock(return_value='[string theory is falsifiable}')
        with patch.object(parallels, 'prlctl', mock_invalid_data):
            snap_name = parallels.snapshot_id_to_name(name, snap_id)
            self.assertEqual(snap_name, '')

        # Data returned from prlctl does not render as a dictionary
        mock_unknown_data = MagicMock(return_value="['sfermions', 'bosinos']")
        with patch.object(parallels, 'prlctl', mock_unknown_data):
            snap_name = parallels.snapshot_id_to_name(name, snap_id)
            self.assertEqual(snap_name, '')

        # Snapshot is unnamed
        mock_no_name = MagicMock(return_value='Name:')
        with patch.object(parallels, 'prlctl', mock_no_name):
            snap_name = parallels.snapshot_id_to_name(name, snap_id)
            self.assertEqual(snap_name, '')

        # If strict, then raise an error when name is not found
        mock_no_name = MagicMock(return_value='Name:')
        with patch.object(parallels, 'prlctl', mock_no_name):
            self.assertRaises(SaltInvocationError,
                              parallels.snapshot_id_to_name,
                              name,
                              snap_id,
                              strict=True)

        # Return name when found
        mock_yes_name = MagicMock(return_value='Name: top')
        with patch.object(parallels, 'prlctl', mock_yes_name):
            snap_name = parallels.snapshot_id_to_name(name, snap_id)
            self.assertEqual(snap_name, 'top')

    def test_snapshot_name_to_id(self):
        '''
        Test parallels.snapshot_name_to_id
        '''
        name = 'macvm'
        snap_ids = ['a5b8999f-5d95-4aff-82de-e515b0101b66',
                    'a7345be5-ab66-478c-946e-a6c2caf14909']
        snap_id = snap_ids[0]
        guid_str = textwrap.dedent('''
            PARENT_SNAPSHOT_ID                      SNAPSHOT_ID
                                                    {a5b8999f-5d95-4aff-82de-e515b0101b66}
            {a5b8999f-5d95-4aff-82de-e515b0101b66} *{a7345be5-ab66-478c-946e-a6c2caf14909}
        ''')
        mock_guids = MagicMock(return_value=guid_str)

        # Raise error when no IDs found for snapshot name
        with patch.object(parallels, 'prlctl', mock_guids):
            mock_no_names = MagicMock(return_value=[])
            with patch.object(parallels, 'snapshot_id_to_name', mock_no_names):
                self.assertRaises(SaltInvocationError,
                                  parallels.snapshot_name_to_id,
                                  name,
                                  'graviton')

        # Validate singly-valued name
        with patch.object(parallels, 'prlctl', mock_guids):
            mock_one_name = MagicMock(side_effect=[u'', u'ν_e'])
            with patch.object(parallels, 'snapshot_id_to_name', mock_one_name):
                self.assertEqual(parallels.snapshot_name_to_id(name, u'ν_e'), snap_ids[1])

        # Validate multiply-valued name
        with patch.object(parallels, 'prlctl', mock_guids):
            mock_many_names = MagicMock(side_effect=[u'J/Ψ', u'J/Ψ'])
            with patch.object(parallels, 'snapshot_id_to_name', mock_many_names):
                self.assertEqual(sorted(parallels.snapshot_name_to_id(name, u'J/Ψ')),
                                 sorted(snap_ids))

        # Raise error for multiply-valued name
        with patch.object(parallels, 'prlctl', mock_guids):
            mock_many_names = MagicMock(side_effect=[u'J/Ψ', u'J/Ψ'])
            with patch.object(parallels, 'snapshot_id_to_name', mock_many_names):
                self.assertRaises(SaltInvocationError,
                                  parallels.snapshot_name_to_id,
                                  name,
                                  u'J/Ψ',
                                  strict=True)

    def test__validate_snap_name(self):
        '''
        Test parallels._validate_snap_name
        '''
        name = 'macvm'
        snap_id = 'a5b8999f-5d95-4aff-82de-e515b0101b66'

        # Validate a GUID passthrough
        self.assertEqual(parallels._validate_snap_name(name, snap_id), snap_id)

        # Validate an unicode name
        mock_snap_symb = MagicMock(return_value=snap_id)
        with patch.object(parallels, 'snapshot_name_to_id', mock_snap_symb):
            self.assertEqual(parallels._validate_snap_name(name, u'π'), snap_id)
            mock_snap_symb.assert_called_once_with(name, u'π', strict=True, runas=None)

        # Validate an ascii name
        mock_snap_name = MagicMock(return_value=snap_id)
        with patch.object(parallels, 'snapshot_name_to_id', mock_snap_name):
            self.assertEqual(parallels._validate_snap_name(name, 'pion'), snap_id)
            mock_snap_name.assert_called_once_with(name, 'pion', strict=True, runas=None)

        # Validate a numerical name
        mock_snap_numb = MagicMock(return_value=snap_id)
        with patch.object(parallels, 'snapshot_name_to_id', mock_snap_numb):
            self.assertEqual(parallels._validate_snap_name(name, '3.14159'), snap_id)
            mock_snap_numb.assert_called_once_with(name, u'3.14159', strict=True, runas=None)

        # Validate not strict (think neutrino oscillation)
        mock_snap_non_strict = MagicMock(return_value=snap_id)
        with patch.object(parallels, 'snapshot_name_to_id', mock_snap_non_strict):
            self.assertEqual(parallels._validate_snap_name(name, u'e_ν', strict=False), snap_id)
            mock_snap_non_strict.assert_called_once_with(name, u'e_ν', strict=False, runas=None)

    def test_list_snapshots(self):
        '''
        Test parallels.list_snapshots
        '''
        name = 'macvm'
        guid_str = textwrap.dedent('''
            PARENT_SNAPSHOT_ID                      SNAPSHOT_ID
                                                    {a5b8999f-5d95-4aff-82de-e515b0101b66}
            {a5b8999f-5d95-4aff-82de-e515b0101b66} *{a7345be5-ab66-478c-946e-a6c2caf14909}
            {a5b8999f-5d95-4aff-82de-e515b0101b66}  {5da9faef-cb0e-466d-9b41-e5571b62ac2a}
        ''')

        # Validate listing all snapshots for the VM
        mock_prlctl = MagicMock(return_value=guid_str)
        with patch.object(parallels, 'prlctl', mock_prlctl):
            parallels.list_snapshots(name)
            mock_prlctl.assert_called_once_with('snapshot-list', [name], runas=None)

        # Validate listing all snapshots in tree mode
        mock_prlctl = MagicMock(return_value=guid_str)
        with patch.object(parallels, 'prlctl', mock_prlctl):
            parallels.list_snapshots(name, tree=True)
            mock_prlctl.assert_called_once_with('snapshot-list', [name, '--tree'], runas=None)

        # Validate listing a single snapshot
        snap_name = 'muon'
        mock_snap_name = MagicMock(return_value=snap_name)
        with patch.object(parallels, '_validate_snap_name', mock_snap_name):
            mock_prlctl = MagicMock(return_value=guid_str)
            with patch.object(parallels, 'prlctl', mock_prlctl):
                parallels.list_snapshots(name, snap_name)
                mock_prlctl.assert_called_once_with('snapshot-list',
                                                    [name, '--id', snap_name],
                                                    runas=None)

        # Validate listing snapshot ID and name pairs
        snap_names = ['electron', 'muon', 'tauon']
        mock_snap_name = MagicMock(side_effect=snap_names)
        with patch.object(parallels, 'snapshot_id_to_name', mock_snap_name):
            mock_prlctl = MagicMock(return_value=guid_str)
            with patch.object(parallels, 'prlctl', mock_prlctl):
                ret = parallels.list_snapshots(name, names=True)
                for snap_name in snap_names:
                    self.assertIn(snap_name, ret)
                mock_prlctl.assert_called_once_with('snapshot-list', [name], runas=None)

    def test_snapshot(self):
        '''
        Test parallels.snapshot
        '''
        name = 'macvm'

        # Validate creating a snapshot
        mock_snap = MagicMock(return_value='')
        with patch.object(parallels, 'prlctl', mock_snap):
            parallels.snapshot(name)
            mock_snap.assert_called_once_with('snapshot', [name], runas=None)

        # Validate creating a snapshot with a name
        snap_name = 'h_0'
        mock_snap_name = MagicMock(return_value='')
        with patch.object(parallels, 'prlctl', mock_snap_name):
            parallels.snapshot(name, snap_name)
            mock_snap_name.assert_called_once_with('snapshot',
                                                   [name, '--name', snap_name],
                                                   runas=None)

        # Validate creating a snapshot with a name and a description
        snap_name = 'h_0'
        snap_desc = textwrap.dedent('The ground state particle of the higgs '
            'multiplet family of bosons')
        mock_snap_name = MagicMock(return_value='')
        with patch.object(parallels, 'prlctl', mock_snap_name):
            parallels.snapshot(name, snap_name, snap_desc)
            mock_snap_name.assert_called_once_with('snapshot',
                                                   [name,
                                                    '--name', snap_name,
                                                    '--description', snap_desc],
                                                   runas=None)

    def test_delete_snapshot(self):
        '''
        Test parallels.delete_snapshot
        '''
        delete_message = ('Delete the snapshot...\n'
                          'The snapshot has been successfully deleted.')

        # Validate single ID
        name = 'macvm'
        snap_name = 'kaon'
        snap_id = 'c2eab062-a635-4ccd-b9ae-998370f898b5'

        mock_snap_name = MagicMock(return_value=snap_id)
        with patch.object(parallels, '_validate_snap_name', mock_snap_name):
            mock_delete = MagicMock(return_value=delete_message)
            with patch.object(parallels, 'prlctl', mock_delete):
                ret = parallels.delete_snapshot(name, snap_name)
                self.assertEqual(ret, delete_message)
                mock_delete.assert_called_once_with('snapshot-delete',
                                                    [name, '--id', snap_id],
                                                    runas=None)

        # Validate multiple IDs
        name = 'macvm'
        snap_name = 'higgs doublet'
        snap_ids = ['c2eab062-a635-4ccd-b9ae-998370f898b5',
                    '8aca07c5-a0e1-4dcb-ba75-cb154d46d516']

        mock_snap_ids = MagicMock(return_value=snap_ids)
        with patch.object(parallels, '_validate_snap_name', mock_snap_ids):
            mock_delete = MagicMock(return_value=delete_message)
            with patch.object(parallels, 'prlctl', mock_delete):
                ret = parallels.delete_snapshot(name, snap_name, all=True)
                mock_ret = {snap_ids[0]: delete_message,
                            snap_ids[1]: delete_message}
                self.assertDictEqual(ret, mock_ret)
                mock_delete.assert_any_call('snapshot-delete',
                                            [name, '--id', snap_ids[0]],
                                            runas=None)
                mock_delete.assert_any_call('snapshot-delete',
                                            [name, '--id', snap_ids[1]],
                                            runas=None)

    def test_revert_snapshot(self):
        '''
        Test parallels.revert_snapshot
        '''
        name = 'macvm'
        snap_name = 'k-bar'
        snap_id = 'c2eab062-a635-4ccd-b9ae-998370f898b5'

        mock_snap_name = MagicMock(return_value=snap_id)
        with patch.object(parallels, '_validate_snap_name', mock_snap_name):
            mock_delete = MagicMock(return_value='')
            with patch.object(parallels, 'prlctl', mock_delete):
                parallels.revert_snapshot(name, snap_name)
                mock_delete.assert_called_once_with('snapshot-switch',
                                                    [name, '--id', snap_id],
                                                    runas=None)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(ParallelsTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Validate the boto_sns module
'''

# Import Python libs
from __future__ import absolute_import
import re

# Import Salt Testing libs
from salttesting import skipIf
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import Salt libs
import integration

# Import 3rd-party libs
NO_BOTO_MODULE = True
BOTO_NOT_CONFIGURED = True
try:
    import boto
    NO_BOTO_MODULE = False
    try:
        boto.connect_iam()
        BOTO_NOT_CONFIGURED = False
    except boto.exception.NoAuthHandlerFound:
        pass
except ImportError:
    pass


@skipIf(
    NO_BOTO_MODULE,
    'Please install the boto library before running boto integration tests.'
)
@skipIf(
    BOTO_NOT_CONFIGURED,
    'Please setup boto AWS credentials before running boto integration tests.'
)
class BotoSNSTest(integration.ModuleCase):

    def setUp(self):
        # The name of the topic you want to create.
        # Constraints: Topic names must be made up of only uppercase and
        # lowercase ASCII letters, numbers, underscores, and hyphens,
        # and must be between 1 and 256 characters long.
        # http://docs.aws.amazon.com/sns/latest/api/API_CreateTopic.html
        self.topic_name = re.sub(r'[^a-zA-Z_-]', '_', self.id())[0:256]
        self.topic_names = [self.topic_name]
        self.run_function('boto_sns.delete', name=self.topic_name)

    def tearDown(self):
        for topic in self.topic_names:
            self.run_function('boto_sns.delete', name=topic)

    def test_exists_non_existing(self):
        ret = self.run_function('boto_sns.exists', ['nonexistent'])
        self.assertSaltModuleFalseReturn(ret)

    def test_exists_existing(self):
        self.run_function('boto_sns.create', [self.topic_name])
        ret = self.run_function('boto_sns.exists', [self.topic_name])
        self.assertSaltModuleTrueReturn(ret)

    def test_create(self):
        ret = self.run_function('boto_sns.create', [self.topic_name])
        self.assertSaltModuleTrueReturn(ret)

        ret = self.run_function('boto_sns.get_all_topics')
        self.assertIn(self.topic_name, list(ret.keys()))
        self.assertIn(self._get_arn(self.topic_name), list(ret.values()))

    def test_delete_non_existing(self):
        ret = self.run_function('boto_sns.delete', [self.topic_name])
        self.assertSaltModuleTrueReturn(ret)

    def test_delete_existing(self):
        self.run_function('boto_sns.create', [self.topic_name])
        ret = self.run_function('boto_sns.delete', [self.topic_name])
        self.assertSaltModuleTrueReturn(ret)

        ret = self.run_function('boto_sns.get_all_topics')
        self.assertNotIn(self.topic_name, list(ret.keys()))
        self.assertNotIn(self._get_arn(self.topic_name), list(ret.values()))

    def test_get_all_topics(self):
        self.topic_names.append(self.topic_name + '-2')
        for topic in self.topic_names:
            self.run_function('boto_sns.create', [topic])

        ret = self.run_function('boto_sns.get_all_topics')

        for topic in self.topic_names:
            self.assertIn(topic, list(ret.keys()))
            self.assertIn(self._get_arn(topic), list(ret.values()))

    def test_subscribe_and_get_all_subscriptions_by_topic(self):
        topic_name = self.topic_name
        ret = self.run_function('boto_sns.create', [topic_name])

        ret = self.run_function(
            'boto_sns.subscribe',
            [topic_name, 'https', 'https://www.example.com/sns/endpoint']
        )
        self.assertSaltModuleTrueReturn(ret)

        ret = self.run_function('boto_sns.get_all_subscriptions_by_topic',
                                [topic_name])
        self.assertDictContainsSubset({
            'Protocol': 'https',
            'Endpoint': 'https://www.example.com/sns/endpoint'
        }, ret[0])

    def _get_arn(self, name):
        return 'arn:aws:sns:us-east-1:{0}:{1}'.format(self.account_id, name)

    @property
    def account_id(self):
        if not hasattr(self, '_account_id'):
            account_id = self.run_function('boto_iam.get_account_id')
            setattr(self, '_account_id', account_id)
        return self._account_id

    def assertSaltModuleTrueReturn(self, ret):
        self.assertIsInstance(ret, bool)
        self.assertTrue(ret)

    def assertSaltModuleFalseReturn(self, ret):
        self.assertIsInstance(ret, bool)
        self.assertFalse(ret)
